@@ -28,7 +28,6 @@
 #include "opensync-archive.h"
 #include "opensync_archive_internals.h"
 #include "opensync-db.h"
-#include <db/opensync_db_internals.h>
 
 /**
  * @defgroup OSyncArchivePrivateAPI OpenSync Archive Internals
@@ -38,11 +37,6 @@
  */
 /*@{*/
 
-void _osync_archive_trace(void *data, const char *query)
-{
-	osync_trace(TRACE_INTERNAL, "query executed: %s", query);
-}
-
 /*@}*/
 
 /**
@@ -137,6 +131,7 @@ static osync_bool osync_archive_create(OSyncDB *db, const char *objtype, OSyncEr
 
 	char *tbl_archive = g_strdup_printf("tbl_archive_%s", objtype);
 	int ret = osync_db_exists(db, tbl_archive, error);
+	g_free(tbl_archive);
 
 	/* error if ret -1 */
 	if (ret < 0)
@@ -205,11 +200,13 @@ error:
  * @brief Increments the reference counter
  * @param archive The pointer to an archive object
  */
-void osync_archive_ref(OSyncArchive *archive)
+OSyncArchive *osync_archive_ref(OSyncArchive *archive)
 {
 	osync_assert(archive);
 	
 	g_atomic_int_inc(&(archive->ref_count));
+
+	return archive;
 }
 
 /**
@@ -240,28 +237,25 @@ void osync_archive_unref(OSyncArchive *archive)
  * @brief Stores data of an entry in the group archive database (blob).
  *
  * @param archive The group archive
- * @param uid UID of requested entry
+ * @param id Archive (database) id of entry to save.
  * @param objtype The object type of the entry
  * @param data The data to store 
  * @param size Total size of data 
  * @param error Pointer to an error struct
  * @return Returns TRUE on success otherwise FALSE
  */ 
-osync_bool osync_archive_save_data(OSyncArchive *archive, const char *uid, const char *objtype, const char *data, unsigned int size, OSyncError **error)
+osync_bool osync_archive_save_data(OSyncArchive *archive, long long int id, const char *objtype, const char *data, unsigned int size, OSyncError **error)
 {
-	osync_trace(TRACE_ENTRY, "%s(%p, %s, %s, %p, %u, %p)", __func__, archive, uid, objtype, data, size, error);
+	osync_trace(TRACE_ENTRY, "%s(%p, %lli, %s, %p, %u, %p)", __func__, archive, id, objtype, data, size, error);
 	osync_assert(archive);
-	osync_assert(uid);
 	osync_assert(data);
 	osync_assert(size);
 
 	if (!osync_archive_create(archive->db, objtype, error))
 		goto error;
 
-	char *escaped_uid = _osync_db_sql_escape(uid);
 	// FIXME: Avoid subselect - this query needs up to 0.5s
-	char *query = g_strdup_printf("REPLACE INTO tbl_archive_%s (mappingid, data) VALUES((SELECT mappingid FROM tbl_changes_%s WHERE uid='%s' LIMIT 1), ?)", objtype, objtype, escaped_uid);
-	g_free(escaped_uid);
+	char *query = g_strdup_printf("REPLACE INTO tbl_archive_%s (mappingid, data) VALUES(%lli, ?)", objtype, id);
 	
 	if (!osync_db_bind_blob(archive->db, query, data, size, error)) {
 		g_free(query);
@@ -300,7 +294,7 @@ int osync_archive_load_data(OSyncArchive *archive, const char *uid, const char *
 	if (!osync_archive_create(archive->db, objtype, error))
 		goto error;
 
-	char *escaped_uid = _osync_db_sql_escape(uid);
+	char *escaped_uid = osync_db_sql_escape(uid);
 	char *query = g_strdup_printf("SELECT data FROM tbl_archive_%s WHERE mappingid=(SELECT mappingid FROM tbl_changes_%s WHERE uid='%s' LIMIT 1)", objtype, objtype, escaped_uid);
 	int ret = osync_db_get_blob(archive->db, query, data, size, error);
 	g_free(query);
@@ -345,7 +339,7 @@ long long int osync_archive_save_change(OSyncArchive *archive, long long int id,
 		goto error;
 
 	char *query = NULL;
-	char *escaped_uid = _osync_db_sql_escape(uid);
+	char *escaped_uid = osync_db_sql_escape(uid);
 
 
 
@@ -470,6 +464,41 @@ error:
 	return FALSE;	
 }
 
+/**
+ * @brief Delete all changes from group archive for a certain object type.
+ *
+ * @param archive The group archive
+ * @param objtype Reported object type of entry
+ * @param error Pointer to an error struct
+ * @return Returns TRUE on success, FALSE otherwise 
+ */ 
+osync_bool osync_archive_flush_changes(OSyncArchive *archive, const char *objtype, OSyncError **error)
+{
+	
+	osync_trace(TRACE_ENTRY, "%s(%p, %s, %p)", __func__, archive, objtype, error);
+	osync_assert(archive);
+	osync_assert(objtype);
+
+	if (!osync_archive_create_changes(archive->db, objtype, error))
+		goto error;
+	
+	char *query = g_strdup_printf("DELETE FROM tbl_changes_%s", objtype);
+	
+	if (!osync_db_query(archive->db, query, error)) {
+		g_free(query);
+		goto error;
+	}
+
+	g_free(query);
+	
+	osync_trace(TRACE_EXIT, "%s", __func__);
+	return TRUE;
+	
+error:
+	osync_trace(TRACE_EXIT_ERROR, "%s: %s", __func__, osync_error_print(error));
+	return FALSE;
+}
+
 /**
  * @brief Loads all conficting changes which were ignored in the previous sync. 
  *
