@@ -31,6 +31,10 @@
 #include "opensync-version.h"
 #include "opensync-merger.h"
 
+#ifdef OPENSYNC_UNITTESTS
+#include "plugin/opensync_plugin_info_internals.h"
+#endif
+
 typedef struct callContext {
 	OSyncClient *client;
 	OSyncMessage *message;
@@ -506,6 +510,12 @@ static osync_bool _osync_client_handle_initialize(OSyncClient *client, OSyncMess
 	osync_plugin_info_set_format_env(client->plugin_info, client->format_env);
 	osync_plugin_info_set_groupname(client->plugin_info, groupname);
 	
+#ifdef OPENSYNC_UNITTESTS
+	long long int memberid;
+	osync_message_read_long_long_int(message, &memberid); // Introduced (only) for testing/debugging purpose (mock-sync)
+	client->plugin_info->memberid = memberid;
+#endif	
+	
 	client->plugin_data = osync_plugin_initialize(client->plugin, client->plugin_info, error);
 	if (!client->plugin_data)
 		goto error;
@@ -1270,6 +1280,11 @@ void osync_client_unref(OSyncClient *client)
 	if (g_atomic_int_dec_and_test(&(client->ref_count))) {
 		osync_trace(TRACE_ENTRY, "%s(%p)", __func__, client);
 		
+		if(client->disconnectThread) {
+			g_thread_join(client->disconnectThread);
+			client->disconnectThread = NULL;
+		}
+
 		if (client->incoming) {
 			if (osync_queue_is_connected(client->incoming))
 				osync_queue_disconnect(client->incoming, NULL);
@@ -1390,7 +1405,9 @@ static gboolean osyncClientDisconnectCallback(gpointer data)
 	if (client->outgoing) {
 		/* We now wait until the other side disconnect our outgoing queue */
 		while (osync_queue_is_connected(client->outgoing)) { g_usleep(100); }
-		
+
+		/* Gives some time if anyone wants to grab the HUP message from the outgoing part of the pipe */
+		g_usleep(200);	
 		/* Now we can safely disconnect our outgoing queue */
 		osync_queue_disconnect(client->outgoing, NULL);
 	}
@@ -1398,17 +1415,31 @@ static gboolean osyncClientDisconnectCallback(gpointer data)
 	return FALSE;
 }
 
-void osync_client_disconnect(OSyncClient *client)
+/* The aim of this thread is to avoid blocking the all the engine event sources with the osyncClientDisconnectCallback internal loop */
+static void client_disconnect_workerthread(gpointer data) 
 {
+	OSyncClient *client = data;
+	GMainContext *context = g_main_context_new();
 	GSource *source = NULL;
-	osync_trace(TRACE_ENTRY, "%s(%p)", __func__, client);
-	
 	source = g_idle_source_new();
 	g_source_set_callback(source, osyncClientDisconnectCallback, client, NULL);
-	g_source_attach(source, client->context);
-	
+	g_source_attach(source, context);
+	OSyncThread *thread = osync_thread_new(context, NULL);
+	osync_thread_start(thread);
+
+	osync_thread_stop(thread);
+	osync_thread_free(thread);
+	thread = NULL;	
 	g_source_unref(source);
-			
+}
+
+	
+void osync_client_disconnect(OSyncClient *client)
+{
+	osync_trace(TRACE_ENTRY, "%s(%p)", __func__, client);
+	
+	client->disconnectThread = g_thread_create((GThreadFunc)client_disconnect_workerthread, client, TRUE, NULL);
+	
 	osync_trace(TRACE_EXIT, "%s", __func__);
 }
 
