@@ -87,6 +87,38 @@ static OSyncObjFormat *_osync_engine_get_internal_format(OSyncEngine *engine, co
 	return osync_format_env_find_objformat(engine->formatenv, format);
 }
 
+static int _osync_engine_get_proxy_position(OSyncEngine *engine, OSyncClientProxy *proxy)
+{
+	osync_assert(engine);
+	osync_assert(proxy);
+
+	int ret = g_list_index(engine->proxies, proxy);
+
+	osync_assert(ret >= 0);
+
+	return ret;
+}
+
+static osync_bool _osync_engine_is_proxy_connected(OSyncEngine *engine, OSyncClientProxy *proxy)
+{
+	osync_assert(engine);
+	osync_assert(proxy);
+
+	return !!(engine->proxy_connects & (1 << _osync_engine_get_proxy_position(engine, proxy)));
+}
+
+static int _osync_engine_get_objengine_position(OSyncEngine *engine, OSyncObjEngine *objengine)
+{
+	osync_assert(engine);
+	osync_assert(objengine);
+
+	int ret = g_list_index(engine->object_engines, objengine);
+
+	osync_assert(ret >= 0);
+
+	return ret;
+}
+
 static void _osync_engine_set_internal_format(OSyncEngine *engine, const char *objtype, OSyncObjFormat *format)
 {
 	osync_trace(TRACE_INTERNAL, "Setting internal format of %s to %p:%s", objtype, format, osync_objformat_get_name(format));
@@ -107,9 +139,10 @@ static void _osync_engine_receive_change(OSyncClientProxy *proxy, void *userdata
 	const char *uid = osync_change_get_uid(change);		
 	int changetype = osync_change_get_changetype(change);
 	const char *format = osync_objformat_get_name(osync_change_get_objformat(change));
+	const char *format_config = osync_objformat_get_config(osync_change_get_objformat(change));
 	const char *objtype = osync_change_get_objtype(change);
 
-	osync_trace(TRACE_INTERNAL, "Received change %s, changetype %i, format %s, objtype %s from member %lli", uid, changetype, format, objtype, memberid);
+	osync_trace(TRACE_INTERNAL, "Received change %s, changetype %i, format %s, format conversion config %s, objtype %s from member %lli", uid, changetype, format, format_config, objtype, memberid);
 	
 	OSyncData *data = osync_change_get_data(change);
 
@@ -118,14 +151,18 @@ static void _osync_engine_receive_change(OSyncClientProxy *proxy, void *userdata
 	osync_trace(TRACE_INTERNAL, "common format %p for objtype %s", internalFormat, osync_change_get_objtype(change));
 
 	/* Only convert if the engine is allowed to convert and if a internal format is available. 
-	   The reason that the engine isn't allowed to convert could be backup. dumping the changes. */
-	if (internalFormat && osync_group_get_converter_enabled(engine->group)) {
+	   The reason that the engine isn't allowed to convert could be backup. dumping the changes. 
+	   Do not convert anything if the chagetype is DELETED. */
+	if (internalFormat && osync_group_get_converter_enabled(engine->group) && (osync_change_get_changetype(change) != OSYNC_CHANGE_TYPE_DELETED)) {
 		osync_trace(TRACE_INTERNAL, "converting to common format %s", osync_objformat_get_name(internalFormat));
 	
-		OSyncFormatConverterPath *path = osync_format_env_find_path(engine->formatenv, osync_change_get_objformat(change), internalFormat, &error);
+		OSyncFormatConverterPath *path = osync_format_env_find_path_with_detectors(engine->formatenv, osync_change_get_data(change), internalFormat, &error);
+
 		if (!path)
 			goto error;
 	
+		osync_converter_path_set_config(path, osync_objformat_get_config(osync_change_get_objformat(change)));
+
 		if (!osync_format_env_convert(engine->formatenv, path, data, &error)) {
 			osync_converter_path_unref(path);
 			goto error;
@@ -194,9 +231,9 @@ static void _osync_engine_receive_change(OSyncClientProxy *proxy, void *userdata
 	return;
 
 error:
+	osync_engine_set_error(engine, error);
 	osync_status_update_member(engine, osync_client_proxy_get_member(proxy), OSYNC_CLIENT_EVENT_ERROR, NULL, error);
 	osync_trace(TRACE_EXIT_ERROR, "%s: %s", __func__, osync_error_print(&error));
-	osync_error_unref(&error);
 }
 
 /* This function is called from the master thread. The function dispatched incoming data from
@@ -496,12 +533,15 @@ static osync_bool _osync_engine_start(OSyncEngine *engine, OSyncError **error)
 {
 	osync_trace(TRACE_ENTRY, "%s(%p, %p)", __func__, engine, error);
 	
-	engine->pluginenv = osync_plugin_env_new(error);
-	if (!engine->pluginenv)
-		goto error;
-	
-	if (!osync_plugin_env_load(engine->pluginenv, engine->plugin_dir, error))
-		goto error;
+	/* For testing purpose, it's possible to preload a instrumented plugin_env */
+	if (!engine->pluginenv) {
+		engine->pluginenv = osync_plugin_env_new(error);
+		if (!engine->pluginenv)
+			goto error;
+		
+		if (!osync_plugin_env_load(engine->pluginenv, engine->plugin_dir, error))
+			goto error;
+	}
 	
 	osync_thread_start(engine->thread);
 
@@ -634,18 +674,26 @@ error:
 
 static osync_bool _osync_engine_generate_connected_event(OSyncEngine *engine)
 {
+	OSyncError *locerror = NULL;
+
 	if (osync_bitcount(engine->proxy_errors | engine->proxy_connects) != g_list_length(engine->proxies))
 		return FALSE;
 	
 	if (osync_bitcount(engine->obj_errors | engine->obj_connects) == g_list_length(engine->object_engines)) {
 		if (osync_bitcount(engine->obj_errors) == g_list_length(engine->object_engines)) {
-			OSyncError *locerror = NULL;
 			osync_error_set(&locerror, OSYNC_ERROR_GENERIC, "No objtypes left without error. Aborting");
 			osync_trace(TRACE_ERROR, "%s", osync_error_print(&locerror));
 			osync_engine_set_error(engine, locerror);
 			osync_status_update_engine(engine, OSYNC_ENGINE_EVENT_ERROR, locerror);
 			osync_engine_event(engine, OSYNC_ENGINE_EVENT_ERROR);
 			osync_error_unref(&locerror);
+		} else if (osync_bitcount(engine->proxy_errors) || osync_bitcount(engine->obj_errors)) {
+			osync_error_set(&locerror, OSYNC_ERROR_GENERIC, "At least one objenit hat problems while connecting. Aborting");
+			osync_trace(TRACE_ERROR, "%s", osync_error_print(&locerror));
+			osync_engine_set_error(engine, locerror);
+			osync_status_update_engine(engine, OSYNC_ENGINE_EVENT_ERROR, locerror);
+			osync_engine_event(engine, OSYNC_ENGINE_EVENT_ERROR);
+			osync_error_unref(&locerror);
 		} else {
 			osync_status_update_engine(engine, OSYNC_ENGINE_EVENT_CONNECTED, NULL);
 			osync_engine_event(engine, OSYNC_ENGINE_EVENT_CONNECTED);
@@ -676,10 +724,19 @@ static void _osync_engine_generate_get_changes_event(OSyncEngine *engine)
 	if (!osync_engine_check_get_changes(engine))
 		return;
 		
-	osync_status_update_engine(engine, OSYNC_ENGINE_EVENT_READ, NULL);
-	osync_status_update_engine(engine, OSYNC_ENGINE_EVENT_END_CONFLICTS, NULL);
-	
-	osync_engine_event(engine, OSYNC_ENGINE_EVENT_READ);
+	if (osync_bitcount(engine->obj_errors)) {
+		OSyncError *locerror = NULL;
+		osync_error_set(&locerror, OSYNC_ERROR_GENERIC, "At least one object engine failed while getting changes. Aborting");
+		osync_trace(TRACE_ERROR, "%s", osync_error_print(&locerror));
+		osync_engine_set_error(engine, locerror);
+		osync_status_update_engine(engine, OSYNC_ENGINE_EVENT_ERROR, locerror);
+		osync_engine_event(engine, OSYNC_ENGINE_EVENT_ERROR);
+	} else {
+		osync_status_update_engine(engine, OSYNC_ENGINE_EVENT_READ, NULL);
+		osync_status_update_engine(engine, OSYNC_ENGINE_EVENT_END_CONFLICTS, NULL);
+		
+		osync_engine_event(engine, OSYNC_ENGINE_EVENT_READ);
+	}
 }
 
 static void _osync_engine_generate_written_event(OSyncEngine *engine)
@@ -688,8 +745,18 @@ static void _osync_engine_generate_written_event(OSyncEngine *engine)
 		return;
 	
 	if (osync_bitcount(engine->obj_errors | engine->obj_written) == g_list_length(engine->object_engines)) {
-		osync_status_update_engine(engine, OSYNC_ENGINE_EVENT_WRITTEN, NULL);
-		osync_engine_event(engine, OSYNC_ENGINE_EVENT_WRITTEN);
+		if (osync_bitcount(engine->obj_errors)) {
+			OSyncError *locerror = NULL;
+			osync_error_set(&locerror, OSYNC_ERROR_GENERIC, "At least one object engine failed while writting changes. Aborting");
+			osync_trace(TRACE_ERROR, "%s", osync_error_print(&locerror));
+			osync_engine_set_error(engine, locerror);
+			osync_status_update_engine(engine, OSYNC_ENGINE_EVENT_ERROR, locerror);
+			osync_engine_event(engine, OSYNC_ENGINE_EVENT_ERROR);
+			//osync_error_unref(&locerror);
+		} else {
+			osync_status_update_engine(engine, OSYNC_ENGINE_EVENT_WRITTEN, NULL);
+			osync_engine_event(engine, OSYNC_ENGINE_EVENT_WRITTEN);
+		}
 	} else
 		osync_trace(TRACE_INTERNAL, "Not yet: %i", osync_bitcount(engine->obj_errors | engine->obj_written));
 
@@ -701,8 +768,17 @@ static void _osync_engine_generate_sync_done_event(OSyncEngine *engine)
 		return;
 	
 	if (osync_bitcount(engine->obj_errors | engine->obj_sync_done) == g_list_length(engine->object_engines)) {
-		osync_status_update_engine(engine, OSYNC_ENGINE_EVENT_SYNC_DONE, NULL);
-		osync_engine_event(engine, OSYNC_ENGINE_EVENT_SYNC_DONE);
+		if (osync_bitcount(engine->obj_errors)) {
+			OSyncError *locerror = NULL;
+			osync_error_set(&locerror, OSYNC_ERROR_GENERIC, "At least one object engine failed within sync_done. Aborting");
+			osync_engine_set_error(engine, locerror);
+			osync_status_update_engine(engine, OSYNC_ENGINE_EVENT_ERROR, locerror);
+			osync_engine_event(engine, OSYNC_ENGINE_EVENT_ERROR);
+			//osync_error_unref(&locerror);
+		} else {
+			osync_status_update_engine(engine, OSYNC_ENGINE_EVENT_SYNC_DONE, NULL);
+			osync_engine_event(engine, OSYNC_ENGINE_EVENT_SYNC_DONE);
+		}
 	} else
 		osync_trace(TRACE_INTERNAL, "Not yet: %i", osync_bitcount(engine->obj_errors | engine->obj_sync_done));
 }
@@ -713,7 +789,12 @@ static osync_bool _osync_engine_generate_disconnected_event(OSyncEngine *engine)
 		return FALSE;
 	
 	if (osync_bitcount(engine->obj_errors | engine->obj_disconnects) == g_list_length(engine->object_engines)) {
-		osync_status_update_engine(engine, OSYNC_ENGINE_EVENT_DISCONNECTED, NULL);
+
+		/* Error handling in this case is quite special. We have to call OSYNC_ENGINE_EVENT_DISCONNECTED,
+		   even on errors. Since OSYNC_ENGINE_EVENT_ERROR would emit this DISCONNECTED event again - deadlock! */
+		if (!osync_bitcount(engine->obj_errors))
+			osync_status_update_engine(engine, OSYNC_ENGINE_EVENT_DISCONNECTED, NULL);
+
 		osync_engine_event(engine, OSYNC_ENGINE_EVENT_DISCONNECTED);
 		return TRUE;
 	}
@@ -722,26 +803,19 @@ static osync_bool _osync_engine_generate_disconnected_event(OSyncEngine *engine)
 	return FALSE;
 }
 
-static void _engine_connect_callback(OSyncClientProxy *proxy, void *userdata, osync_bool slowsync, OSyncError *error)
+static void _osync_engine_connect_callback(OSyncClientProxy *proxy, void *userdata, osync_bool slowsync, OSyncError *error)
 {
 	OSyncEngine *engine = userdata;
 	osync_trace(TRACE_ENTRY, "%s(%p, %p, %i, %p)", __func__, proxy, userdata, slowsync, error);
 	
-	int i = 0;
-	GList *e = NULL;
-	for (e = engine->proxies; e; e = e->next) {
-		if (e->data == proxy)
-			break;
-		i++;
-	}
+	int position = _osync_engine_get_proxy_position(engine, proxy);
 	
 	if (error) {
-		engine->proxy_errors = engine->proxy_errors | (0x1 << i);
-		engine->error = error;
-		osync_error_ref(&error);
+		osync_engine_set_error(engine, error);
+		engine->proxy_errors = engine->proxy_errors | (0x1 << position);
 		osync_status_update_member(engine, osync_client_proxy_get_member(proxy), OSYNC_CLIENT_EVENT_ERROR, NULL, error);
 	} else {
-		engine->proxy_connects = engine->proxy_connects | (0x1 << i);
+		engine->proxy_connects = engine->proxy_connects | (0x1 << position);
 		osync_status_update_member(engine, osync_client_proxy_get_member(proxy), OSYNC_CLIENT_EVENT_CONNECTED, NULL, NULL);
 	}
 	
@@ -750,26 +824,19 @@ static void _engine_connect_callback(OSyncClientProxy *proxy, void *userdata, os
 	osync_trace(TRACE_EXIT, "%s", __func__);
 }
 
-static void _engine_disconnect_callback(OSyncClientProxy *proxy, void *userdata, OSyncError *error)
+static void _osync_engine_disconnect_callback(OSyncClientProxy *proxy, void *userdata, OSyncError *error)
 {
 	OSyncEngine *engine = userdata;
 	osync_trace(TRACE_ENTRY, "%s(%p, %p, %p)", __func__, proxy, userdata, error);
 	
-	int i = 0;
-	GList *e = NULL;
-	for (e = engine->proxies; e; e = e->next) {
-		if (e->data == proxy)
-			break;
-		i++;
-	}
+	int position = _osync_engine_get_proxy_position(engine, proxy);
 	
 	if (error) {
-		engine->proxy_errors = engine->proxy_errors | (0x1 << i);
-		engine->error = error;
-		osync_error_ref(&error);
+		osync_engine_set_error(engine, error);
+		engine->proxy_errors = engine->proxy_errors | (0x1 << position);
 		osync_status_update_member(engine, osync_client_proxy_get_member(proxy), OSYNC_CLIENT_EVENT_ERROR, NULL, error);
 	} else {
-		engine->proxy_disconnects = engine->proxy_disconnects | (0x1 << i);
+		engine->proxy_disconnects = engine->proxy_disconnects | (0x1 << position);
 		osync_status_update_member(engine, osync_client_proxy_get_member(proxy), OSYNC_CLIENT_EVENT_DISCONNECTED, NULL, NULL);
 	}
 	
@@ -778,26 +845,19 @@ static void _engine_disconnect_callback(OSyncClientProxy *proxy, void *userdata,
 	osync_trace(TRACE_EXIT, "%s", __func__);
 }
 
-static void _engine_get_changes_callback(OSyncClientProxy *proxy, void *userdata, OSyncError *error)
+static void _osync_engine_get_changes_callback(OSyncClientProxy *proxy, void *userdata, OSyncError *error)
 {
 	OSyncEngine *engine = userdata;
 	osync_trace(TRACE_ENTRY, "%s(%p, %p, %p)", __func__, proxy, userdata, error);
 	
-	int i = 0;
-	GList *e = NULL;
-	for (e = engine->proxies; e; e = e->next) {
-		if (e->data == proxy)
-			break;
-		i++;
-	}
+	int position = _osync_engine_get_proxy_position(engine, proxy);
 	
 	if (error) {
-		engine->proxy_errors = engine->proxy_errors | (0x1 << i);
-		engine->error = error;
-		osync_error_ref(&error);
+		osync_engine_set_error(engine, error);
+		engine->proxy_errors = engine->proxy_errors | (0x1 << position);
 		osync_status_update_member(engine, osync_client_proxy_get_member(proxy), OSYNC_CLIENT_EVENT_ERROR, NULL, error);
 	} else {
-		engine->proxy_get_changes = engine->proxy_get_changes | (0x1 << i);
+		engine->proxy_get_changes = engine->proxy_get_changes | (0x1 << position);
 		osync_status_update_member(engine, osync_client_proxy_get_member(proxy), OSYNC_CLIENT_EVENT_READ, NULL, NULL);
 	}
 	
@@ -806,26 +866,19 @@ static void _engine_get_changes_callback(OSyncClientProxy *proxy, void *userdata
 	osync_trace(TRACE_EXIT, "%s", __func__);
 }
 
-static void _engine_written_callback(OSyncClientProxy *proxy, void *userdata, OSyncError *error)
+static void _osync_engine_written_callback(OSyncClientProxy *proxy, void *userdata, OSyncError *error)
 {
 	OSyncEngine *engine = userdata;
 	osync_trace(TRACE_ENTRY, "%s(%p, %p, %p)", __func__, proxy, userdata, error);
 	
-	int i = 0;
-	GList *e = NULL;
-	for (e = engine->proxies; e; e = e->next) {
-		if (e->data == proxy)
-			break;
-		i++;
-	}
+	int position = _osync_engine_get_proxy_position(engine, proxy);
 	
 	if (error) {
-		engine->proxy_errors = engine->proxy_errors | (0x1 << i);
-		engine->error = error;
-		osync_error_ref(&error);
+		osync_engine_set_error(engine, error);
+		engine->proxy_errors = engine->proxy_errors | (0x1 << position);
 		osync_status_update_member(engine, osync_client_proxy_get_member(proxy), OSYNC_CLIENT_EVENT_ERROR, NULL, error);
 	} else {
-		engine->proxy_written = engine->proxy_written | (0x1 << i);
+		engine->proxy_written = engine->proxy_written | (0x1 << position);
 		osync_status_update_member(engine, osync_client_proxy_get_member(proxy), OSYNC_CLIENT_EVENT_WRITTEN, NULL, NULL);
 	}
 	
@@ -834,26 +887,19 @@ static void _engine_written_callback(OSyncClientProxy *proxy, void *userdata, OS
 	osync_trace(TRACE_EXIT, "%s", __func__);
 }
 
-static void _engine_sync_done_callback(OSyncClientProxy *proxy, void *userdata, OSyncError *error)
+static void _osync_engine_sync_done_callback(OSyncClientProxy *proxy, void *userdata, OSyncError *error)
 {
 	OSyncEngine *engine = userdata;
 	osync_trace(TRACE_ENTRY, "%s(%p, %p, %p)", __func__, proxy, userdata, error);
 	
-	int i = 0;
-	GList *e = NULL;
-	for (e = engine->proxies; e; e = e->next) {
-		if (e->data == proxy)
-			break;
-		i++;
-	}
+	int position = _osync_engine_get_proxy_position(engine, proxy);
 	
 	if (error) {
-		engine->proxy_errors = engine->proxy_errors | (0x1 << i);
-		engine->error = error;
-		osync_error_ref(&error);
+		osync_engine_set_error(engine, error);
+		engine->proxy_errors = engine->proxy_errors | (0x1 << position);
 		osync_status_update_member(engine, osync_client_proxy_get_member(proxy), OSYNC_CLIENT_EVENT_ERROR, NULL, error);
 	} else {
-		engine->proxy_sync_done = engine->proxy_sync_done | (0x1 << i);
+		engine->proxy_sync_done = engine->proxy_sync_done | (0x1 << position);
 		osync_status_update_member(engine, osync_client_proxy_get_member(proxy), OSYNC_CLIENT_EVENT_SYNC_DONE, NULL, NULL);
 	}
 	
@@ -862,62 +908,93 @@ static void _engine_sync_done_callback(OSyncClientProxy *proxy, void *userdata,
 	osync_trace(TRACE_EXIT, "%s", __func__);
 }
 
-static void _engine_event_callback(OSyncObjEngine *objengine, OSyncEngineEvent event, OSyncError *error, void *userdata)
+static void _osync_engine_get_objengine_error(OSyncEngine *engine, OSyncObjEngine *objengine, int position, OSyncError *error)
+{
+	engine->obj_errors = engine->obj_errors | (0x1 << position);
+	osync_engine_set_error(engine, error);
+}
+
+static void _osync_engine_get_objengine_event(OSyncEngine *engine, OSyncObjEngine *objengine, int position, OSyncEngineEvent event)
 {
-	OSyncEngine *engine = userdata;
-	osync_trace(TRACE_ENTRY, "%s(%p, %i, %p, %p)", __func__, objengine, event, error, userdata);
-	
-	int i = 0;
-	GList *e = NULL;
-	for (e = engine->object_engines; e; e = e->next) {
-		if (e->data == objengine)
-			break;
-		i++;
-	}
-	
 	switch (event) {
 		case OSYNC_ENGINE_EVENT_CONNECTED:
-			engine->obj_connects = engine->obj_connects | (0x1 << i);
-			_osync_engine_generate_connected_event(engine);
+			engine->obj_connects = engine->obj_connects | (0x1 << position);
 			break;
 		case OSYNC_ENGINE_EVENT_ERROR:
-			osync_trace(TRACE_ERROR, "Engine received an error: %s", osync_error_print(&error));
-			osync_engine_set_error(engine, error);
-			engine->obj_errors = engine->obj_errors | (0x1 << i);
+			/* ObjEngine don't emit this signal. To determine which actual event fail,
+			   the ObjEngine emits regular event type and pass an OSyncError object.
+			   See _osync_engine_generate_event() and _osync_engine_get_obj_engine_error(). */
 			break;
 		case OSYNC_ENGINE_EVENT_READ:
-			engine->obj_get_changes = engine->obj_get_changes | (0x1 << i);
-			_osync_engine_generate_get_changes_event(engine);
+			engine->obj_get_changes = engine->obj_get_changes | (0x1 << position);
 			break;
 		case OSYNC_ENGINE_EVENT_WRITTEN:
-			engine->obj_written = engine->obj_written | (0x1 << i);
-			_osync_engine_generate_written_event(engine);
+			engine->obj_written = engine->obj_written | (0x1 << position);
 			break;
 		case OSYNC_ENGINE_EVENT_SYNC_DONE:
-			engine->obj_sync_done = engine->obj_sync_done | (0x1 << i);
-			_osync_engine_generate_sync_done_event(engine);
+			engine->obj_sync_done = engine->obj_sync_done | (0x1 << position);
 			break;
 		case OSYNC_ENGINE_EVENT_DISCONNECTED:
-			engine->obj_disconnects = engine->obj_disconnects | (0x1 << i);
-			_osync_engine_generate_disconnected_event(engine);
+			engine->obj_disconnects = engine->obj_disconnects | (0x1 << position);
 			break;
 		case OSYNC_ENGINE_EVENT_SUCCESSFUL:
 		case OSYNC_ENGINE_EVENT_END_CONFLICTS:
 		case OSYNC_ENGINE_EVENT_PREV_UNCLEAN:
 			break;
 	}
+}
+
+static void _osync_engine_generate_event(OSyncEngine *engine, OSyncEngineEvent event)
+{
+	switch (event) {
+		case OSYNC_ENGINE_EVENT_CONNECTED:
+			_osync_engine_generate_connected_event(engine);
+			break;
+		case OSYNC_ENGINE_EVENT_READ:
+			_osync_engine_generate_get_changes_event(engine);
+			break;
+		case OSYNC_ENGINE_EVENT_WRITTEN:
+			_osync_engine_generate_written_event(engine);
+			break;
+		case OSYNC_ENGINE_EVENT_SYNC_DONE:
+			_osync_engine_generate_sync_done_event(engine);
+			break;
+		case OSYNC_ENGINE_EVENT_DISCONNECTED:
+			_osync_engine_generate_disconnected_event(engine);
+			break;
+		case OSYNC_ENGINE_EVENT_ERROR:
+		case OSYNC_ENGINE_EVENT_SUCCESSFUL:
+		case OSYNC_ENGINE_EVENT_END_CONFLICTS:
+		case OSYNC_ENGINE_EVENT_PREV_UNCLEAN:
+			break;
+	}
+}
+
+static void _osync_engine_event_callback(OSyncObjEngine *objengine, OSyncEngineEvent event, OSyncError *error, void *userdata)
+{
+	OSyncError *locerror = NULL;
+	OSyncEngine *engine = userdata;
+	osync_trace(TRACE_ENTRY, "%s(%p, %i, %p, %p)", __func__, objengine, event, error, userdata);
+
+	int position = _osync_engine_get_objengine_position(engine, objengine);
 	
+	if (error)
+		_osync_engine_get_objengine_error(engine, objengine, position, error);
+	else
+		_osync_engine_get_objengine_event(engine, objengine, position, event);
+
+	_osync_engine_generate_event(engine, event);
+
 	osync_trace(TRACE_EXIT, "%s", __func__);
 }
 
-static void _engine_discover_callback(OSyncClientProxy *proxy, void *userdata, OSyncError *error)
+static void _osync_engine_discover_callback(OSyncClientProxy *proxy, void *userdata, OSyncError *error)
 {
 	OSyncEngine *engine = userdata;
 	osync_trace(TRACE_ENTRY, "%s(%p, %p, %p)", __func__, proxy, userdata, error);
 	
 	if (error) {
-		engine->error = error;
-		osync_error_ref(&error);
+		osync_engine_set_error(engine, error);
 		osync_status_update_member(engine, osync_client_proxy_get_member(proxy), OSYNC_CLIENT_EVENT_ERROR, NULL, error);
 	} else {
 		osync_status_update_member(engine, osync_client_proxy_get_member(proxy), OSYNC_CLIENT_EVENT_DISCOVERED, NULL, NULL);
@@ -997,7 +1074,7 @@ osync_bool osync_engine_initialize(OSyncEngine *engine, OSyncError **error)
 	/* Lets see which objtypes are synchronizable in this group */
 	int num = osync_group_num_objtypes(engine->group);
 	if (num == 0) {
-		osync_error_set(&engine->error, OSYNC_ERROR_GENERIC, "No synchronizable objtype");
+		osync_error_set(error, OSYNC_ERROR_GENERIC, "No synchronizable objtype");
 		goto error;
 	}
 
@@ -1008,11 +1085,11 @@ osync_bool osync_engine_initialize(OSyncEngine *engine, OSyncError **error)
 		if (!osync_group_objtype_enabled(engine->group, objtype))
 			continue;
 
-		OSyncObjEngine *objengine = osync_obj_engine_new(engine, objtype, engine->formatenv, &engine->error);
+		OSyncObjEngine *objengine = osync_obj_engine_new(engine, objtype, engine->formatenv, error);
 		if (!objengine)
 			goto error;
 
-		osync_obj_engine_set_callback(objengine, _engine_event_callback, engine);
+		osync_obj_engine_set_callback(objengine, _osync_engine_event_callback, engine);
 		engine->object_engines = g_list_append(engine->object_engines, objengine);
 
 		/* If previous sync was unclean, then trigger SlowSync for all ObjEngines */
@@ -1057,14 +1134,20 @@ osync_bool osync_engine_finalize(OSyncEngine *engine, OSyncError **error)
 	
 	_osync_engine_stop(engine);
 	
-	if (engine->formatenv)
+	if (engine->formatenv) {
 		osync_format_env_free(engine->formatenv);
+		engine->formatenv = NULL;
+	}
 	
-	if (engine->pluginenv)
+	if (engine->pluginenv) {
 		osync_plugin_env_free(engine->pluginenv);
+		engine->pluginenv = NULL;
+	}
 
-	osync_group_unlock(engine->group);
+	if (!engine->error)
+		osync_group_unlock(engine->group);
 
+	osync_error_unref(&(engine->error));
 	
 	osync_trace(TRACE_EXIT, "%s", __func__);
 	return TRUE;
@@ -1078,6 +1161,7 @@ void osync_engine_command(OSyncEngine *engine, OSyncEngineCommand *command)
 {
 	GList *o = NULL;
 	GList *p = NULL;
+	OSyncError *locerror = NULL;
 	OSyncClientProxy *proxy = NULL;
 			
 	osync_trace(TRACE_ENTRY, "%s(%p, %p)", __func__, engine, command);
@@ -1090,17 +1174,17 @@ void osync_engine_command(OSyncEngine *engine, OSyncEngineCommand *command)
 			for (o = engine->object_engines; o; o = o->next) {
 				OSyncObjEngine *objengine = o->data;
 
-				if (!osync_obj_engine_initialize(objengine, &engine->error))
+				if (!osync_obj_engine_initialize(objengine, &locerror))
 					goto error;
 
-				if (!osync_obj_engine_command(objengine, OSYNC_ENGINE_COMMAND_CONNECT, &engine->error))
+				if (!osync_obj_engine_command(objengine, OSYNC_ENGINE_COMMAND_CONNECT, &locerror))
 					goto error;
 			}
 			
-			/* Then we connect ourselves */
+			/* Then we connect the main sinks */
 			for (o = engine->proxies; o; o = o->next) {
 				OSyncClientProxy *proxy = o->data;
-				if (!osync_client_proxy_connect(proxy, _engine_connect_callback, engine, NULL, FALSE, &engine->error))
+				if (!osync_client_proxy_connect(proxy, _osync_engine_connect_callback, engine, NULL, FALSE, &locerror))
 					goto error;
 			}
 			break;
@@ -1112,19 +1196,19 @@ void osync_engine_command(OSyncEngine *engine, OSyncEngineCommand *command)
 		case OSYNC_ENGINE_COMMAND_SOLVE:
 			switch (command->solve_type) {
 				case OSYNC_ENGINE_SOLVE_CHOOSE:
-					if (!osync_mapping_engine_solve(command->mapping_engine, command->master, &engine->error))
+					if (!osync_mapping_engine_solve(command->mapping_engine, command->master, &locerror))
 						goto error;
 					break;
 				case OSYNC_ENGINE_SOLVE_DUPLICATE:
-					if (!osync_mapping_engine_duplicate(command->mapping_engine, &engine->error))
+					if (!osync_mapping_engine_duplicate(command->mapping_engine, &locerror))
 						goto error;
 					break;
 				case OSYNC_ENGINE_SOLVE_IGNORE:
-					if (!osync_mapping_engine_ignore(command->mapping_engine, &engine->error))
+					if (!osync_mapping_engine_ignore(command->mapping_engine, &locerror))
 						goto error;
 					break;
 				case OSYNC_ENGINE_SOLVE_USE_LATEST:
-					if (!osync_mapping_engine_use_latest(command->mapping_engine, &engine->error))
+					if (!osync_mapping_engine_use_latest(command->mapping_engine, &locerror))
 						goto error;
 					break;
 			}
@@ -1138,11 +1222,11 @@ void osync_engine_command(OSyncEngine *engine, OSyncEngineCommand *command)
 			}
 			
 			if (!proxy) {
-				osync_error_set(&engine->error, OSYNC_ERROR_GENERIC, "Unable to find member");
+				osync_error_set(&locerror, OSYNC_ERROR_GENERIC, "Unable to find member");
 				goto error;
 			}
 		
-			if (!osync_client_proxy_discover(proxy, _engine_discover_callback, engine, &engine->error))
+			if (!osync_client_proxy_discover(proxy, _osync_engine_discover_callback, engine, &locerror))
 				goto error;
 		
 			break;
@@ -1152,81 +1236,99 @@ void osync_engine_command(OSyncEngine *engine, OSyncEngineCommand *command)
 	return;
 	
 error:
+	osync_engine_set_error(engine, locerror);
+
 	g_mutex_lock(engine->syncing_mutex);
 	g_cond_signal(engine->syncing);
 	g_mutex_unlock(engine->syncing_mutex);
 
-	osync_trace(TRACE_EXIT_ERROR, "%s: %s", __func__, osync_error_print(&engine->error));
-	osync_error_ref(&engine->error);
+	osync_trace(TRACE_EXIT_ERROR, "%s: %s", __func__, osync_error_print(&locerror));
 }
 
 void osync_engine_event(OSyncEngine *engine, OSyncEngineEvent event)
 {
 	GList *o = NULL;
+	OSyncError *locerror = NULL;
 	
 	osync_trace(TRACE_ENTRY, "%s(%p, %i)", __func__, engine, event);
 	osync_assert(engine);
 	
+
 	switch (event) {
 		case OSYNC_ENGINE_EVENT_CONNECTED:
 			/* Now that we are connected, we read the changes */
 			for (o = engine->object_engines; o; o = o->next) {
 				OSyncObjEngine *objengine = o->data;
-				if (!osync_obj_engine_command(objengine, OSYNC_ENGINE_COMMAND_READ, &engine->error))
+				if (!osync_obj_engine_command(objengine, OSYNC_ENGINE_COMMAND_READ, &locerror))
 					goto error;
 			}
 			
 			/* Now we read the main sink */
 			for (o = engine->proxies; o; o = o->next) {
 				OSyncClientProxy *proxy = o->data;
-				if (!osync_client_proxy_get_changes(proxy, _engine_get_changes_callback, engine, NULL, FALSE, &engine->error))
+				if (!osync_client_proxy_get_changes(proxy, _osync_engine_get_changes_callback, engine, NULL, FALSE, &locerror))
 					goto error;
 			}
+
 			break;
 		case OSYNC_ENGINE_EVENT_READ:
 			/* Now that we are connected, we write the changes */
 			for (o = engine->object_engines; o; o = o->next) {
 				OSyncObjEngine *objengine = o->data;
-				if (!osync_obj_engine_command(objengine, OSYNC_ENGINE_COMMAND_WRITE, &engine->error))
+				if (!osync_obj_engine_command(objengine, OSYNC_ENGINE_COMMAND_WRITE, &locerror))
 					goto error;
 			}
-			
+
 			/* Now we write the main sink */
 			for (o = engine->proxies; o; o = o->next) {
 				OSyncClientProxy *proxy = o->data;
-				if (!osync_client_proxy_committed_all(proxy, _engine_written_callback, engine, NULL, &engine->error))
+				if (!osync_client_proxy_committed_all(proxy, _osync_engine_written_callback, engine, NULL, &locerror))
 					goto error;
 			}
+
 			break;
 		case OSYNC_ENGINE_EVENT_WRITTEN:
 			/* Lets call sync done */
 			for (o = engine->object_engines; o; o = o->next) {
 				OSyncObjEngine *objengine = o->data;
-				if (!osync_obj_engine_command(objengine, OSYNC_ENGINE_COMMAND_SYNC_DONE, &engine->error))
+				if (!osync_obj_engine_command(objengine, OSYNC_ENGINE_COMMAND_SYNC_DONE, &locerror))
 					goto error;
 			}
 			
 			/* Now we call sync done on the main sink */
 			for (o = engine->proxies; o; o = o->next) {
 				OSyncClientProxy *proxy = o->data;
-				if (!osync_client_proxy_sync_done(proxy, _engine_sync_done_callback, engine, NULL, &engine->error))
+				if (!osync_client_proxy_sync_done(proxy, _osync_engine_sync_done_callback, engine, NULL, &locerror))
 					goto error;
 			}
+
 			break;
+		case OSYNC_ENGINE_EVENT_ERROR:
+			osync_trace(TRACE_ERROR, "Engine aborting due to an error: %s", osync_error_print(&(engine->error)));
 		case OSYNC_ENGINE_EVENT_SYNC_DONE:
 			/* Lets disconnect */
 			for (o = engine->object_engines; o; o = o->next) {
 				OSyncObjEngine *objengine = o->data;
-				if (!osync_obj_engine_command(objengine, OSYNC_ENGINE_COMMAND_DISCONNECT, &engine->error))
+				if (!osync_obj_engine_command(objengine, OSYNC_ENGINE_COMMAND_DISCONNECT, &locerror))
 					goto error;
 			}
-			
+
 			/* Now we disconnect the main sink */
 			for (o = engine->proxies; o; o = o->next) {
 				OSyncClientProxy *proxy = o->data;
-				if (!osync_client_proxy_disconnect(proxy, _engine_disconnect_callback, engine, NULL, &engine->error))
+
+				if (!_osync_engine_is_proxy_connected(engine, proxy)) {
+					_osync_engine_disconnect_callback(proxy, engine, NULL);
+					continue;
+				}
+
+				if (!osync_client_proxy_disconnect(proxy, _osync_engine_disconnect_callback, engine, NULL, &locerror))
 					goto error;
 			}
+			
+			if (!engine->error)
+				osync_status_update_engine(engine, OSYNC_ENGINE_EVENT_SUCCESSFUL, NULL);
+
 			break;
 		case OSYNC_ENGINE_EVENT_DISCONNECTED:
 
@@ -1249,15 +1351,6 @@ void osync_engine_event(OSyncEngine *engine, OSyncEngineEvent event)
 			engine->obj_written = 0;
 			engine->obj_sync_done = 0;
 			
-			osync_status_update_engine(engine, OSYNC_ENGINE_EVENT_SUCCESSFUL, NULL);
-			
-			g_mutex_lock(engine->syncing_mutex);
-			g_cond_signal(engine->syncing);
-			g_mutex_unlock(engine->syncing_mutex);
-			break;
-		case OSYNC_ENGINE_EVENT_ERROR:
-			osync_trace(TRACE_ERROR, "Engine aborting due to an error: %s", osync_error_print(&(engine->error)));
-			
 			g_mutex_lock(engine->syncing_mutex);
 			g_cond_signal(engine->syncing);
 			g_mutex_unlock(engine->syncing_mutex);
@@ -1272,12 +1365,13 @@ void osync_engine_event(OSyncEngine *engine, OSyncEngineEvent event)
 	return;
 	
 error:
+	osync_engine_set_error(engine, locerror);
+
 	g_mutex_lock(engine->syncing_mutex);
 	g_cond_signal(engine->syncing);
 	g_mutex_unlock(engine->syncing_mutex);
 
-	osync_trace(TRACE_EXIT_ERROR, "%s: %s", __func__, osync_error_print(&engine->error));
-	osync_error_ref(&engine->error);
+	osync_trace(TRACE_EXIT_ERROR, "%s: %s", __func__, osync_error_print(&locerror));
 }
 
 /*! @brief Starts to synchronize the given OSyncEngine
@@ -1345,8 +1439,6 @@ osync_bool osync_engine_synchronize_and_block(OSyncEngine *engine, OSyncError **
 		osync_trace(TRACE_ERROR, "error while synchronizing: %s", msg);
 		g_free(msg);
 		osync_error_set_from_error(error, &(engine->error));
-		osync_error_unref(&(engine->error));
-		engine->error = NULL;
 		goto error;
 	}
 	
