@@ -75,7 +75,7 @@ error:
 	return NULL;
 }
 
-static void _obj_engine_connect_callback(OSyncClientProxy *proxy, void *userdata, osync_bool slowsync, OSyncError *error)
+static void _osync_obj_engine_connect_callback(OSyncClientProxy *proxy, void *userdata, osync_bool slowsync, OSyncError *error)
 {
 	OSyncSinkEngine *sinkengine = userdata;
 	OSyncObjEngine *engine = sinkengine->engine;
@@ -102,22 +102,45 @@ static void _obj_engine_connect_callback(OSyncClientProxy *proxy, void *userdata
 		if (osync_bitcount(engine->sink_connects) < 2) {
 			osync_error_set(&locerror, OSYNC_ERROR_GENERIC, "Less than 2 sink_engines are connected");
 			osync_obj_engine_set_error(engine, locerror);
-			
-			osync_obj_engine_event(engine, OSYNC_ENGINE_EVENT_ERROR);
-		} else {
-			osync_obj_engine_event(engine, OSYNC_ENGINE_EVENT_CONNECTED);
+		} else if (osync_bitcount(engine->sink_errors)) {
+			osync_error_set(&locerror, OSYNC_ERROR_GENERIC, "At least one sink_engine failed while connecting");
+			osync_obj_engine_set_error(engine, locerror);
 		}
+
+		osync_obj_engine_event(engine, OSYNC_ENGINE_EVENT_CONNECTED, locerror ? locerror : error);
 	} else
 		osync_trace(TRACE_INTERNAL, "Not yet: %i", osync_bitcount(engine->sink_errors | engine->sink_connects));
 	
 	osync_trace(TRACE_EXIT, "%s", __func__);
 }
 
-static void _obj_engine_disconnect_callback(OSyncClientProxy *proxy, void *userdata, OSyncError *error)
+static void _osync_obj_engine_generate_event_disconnected(OSyncObjEngine *engine, OSyncError *error)
+{
+	osync_trace(TRACE_ENTRY, "%s(%p)", __func__, engine);
+
+	OSyncError *locerror = NULL;
+
+	if (osync_bitcount(engine->sink_errors | engine->sink_disconnects) == g_list_length(engine->sink_engines)) {
+		if (osync_bitcount(engine->sink_disconnects) < osync_bitcount(engine->sink_connects)) {
+			osync_error_set(&locerror, OSYNC_ERROR_GENERIC, "Fewer sink_engines disconnected than connected");
+			osync_obj_engine_set_error(engine, locerror);
+			osync_error_unref(&locerror);
+		}
+
+		/* Since disconnect errors don't affect the data integrity keep the sync successful, even on
+		   a disconnect error. So we have to avoid OSyncEngine->error got set, 
+		   just keep this ObjEngine disconnect errors at this engine. */
+		osync_obj_engine_event(engine, OSYNC_ENGINE_EVENT_DISCONNECTED, NULL);
+	} else
+		osync_trace(TRACE_INTERNAL, "Not yet: %i", osync_bitcount(engine->sink_errors | engine->sink_disconnects));
+
+	osync_trace(TRACE_EXIT, "%s", __func__);
+}
+
+static void _osync_obj_engine_disconnect_callback(OSyncClientProxy *proxy, void *userdata, OSyncError *error)
 {
 	OSyncSinkEngine *sinkengine = userdata;
 	OSyncObjEngine *engine = sinkengine->engine;
-	OSyncError *locerror = NULL;
 	
 	osync_trace(TRACE_ENTRY, "%s(%p, %p, %p)", __func__, proxy, userdata, error);
 	
@@ -129,17 +152,9 @@ static void _obj_engine_disconnect_callback(OSyncClientProxy *proxy, void *userd
 		engine->sink_disconnects = engine->sink_disconnects | (0x1 << sinkengine->position);
 		osync_status_update_member(engine->parent, osync_client_proxy_get_member(proxy), OSYNC_CLIENT_EVENT_DISCONNECTED, engine->objtype, NULL);
 	}
-			
-	if (osync_bitcount(engine->sink_errors | engine->sink_disconnects) == g_list_length(engine->sink_engines)) {
-		if (osync_bitcount(engine->sink_disconnects) < osync_bitcount(engine->sink_connects)) {
-			osync_error_set(&locerror, OSYNC_ERROR_GENERIC, "Fewer sink_engines disconnected than connected");
-			osync_obj_engine_set_error(engine, locerror);
-			osync_obj_engine_event(engine, OSYNC_ENGINE_EVENT_ERROR);
-		} else
-			osync_obj_engine_event(engine, OSYNC_ENGINE_EVENT_DISCONNECTED);
-	} else
-		osync_trace(TRACE_INTERNAL, "Not yet: %i", osync_bitcount(engine->sink_errors | engine->sink_disconnects));
 	
+	_osync_obj_engine_generate_event_disconnected(engine, error);
+
 	osync_trace(TRACE_EXIT, "%s", __func__);
 }
 
@@ -147,7 +162,7 @@ static void _obj_engine_disconnect_callback(OSyncClientProxy *proxy, void *userd
  * return value is MISMATCH if no mapping could be found,
  * SIMILAR if a mapping has been found but its not completely the same
  * SAME if a mapping has been found and is the same */
-static OSyncConvCmpResult _obj_engine_mapping_find(OSyncObjEngine *engine, OSyncChange *change, OSyncSinkEngine *sinkengine, OSyncMappingEngine **mapping_engine)
+static OSyncConvCmpResult _osync_obj_engine_mapping_find(OSyncObjEngine *engine, OSyncChange *change, OSyncSinkEngine *sinkengine, OSyncMappingEngine **mapping_engine)
 {	
 	GList *m = NULL;
 	GList *e = NULL;
@@ -216,7 +231,7 @@ osync_bool osync_obj_engine_map_changes(OSyncObjEngine *engine, OSyncError **err
 	
 			
 			/* See if there is an exisiting mapping, which fits the unmapped change */
-			OSyncConvCmpResult result = _obj_engine_mapping_find(engine, change, sinkengine, &mapping_engine);
+			OSyncConvCmpResult result = _osync_obj_engine_mapping_find(engine, change, sinkengine, &mapping_engine);
 			if (result == OSYNC_CONV_DATA_MISMATCH) {
 				/* If there is none, create one */
 				mapping_engine = _osync_obj_engine_create_mapping_engine(engine, error);
@@ -251,13 +266,15 @@ error:
 	return FALSE;
 }
 
-static void _obj_engine_read_ignored_callback(OSyncClientProxy *proxy, void *userdata, OSyncError *error)
+static void _osync_obj_engine_read_ignored_callback(OSyncClientProxy *proxy, void *userdata, OSyncError *error)
 {
-	// NOOP	
+	/* TODO: Share _generate_read_event fucntion with _osync_obj_engine_read_callback?
+	   To report errors .. and handle _timeout problems of _read_ignored call.
+	 */
 }
 
 
-static void _obj_engine_read_callback(OSyncClientProxy *proxy, void *userdata, OSyncError *error)
+static void _osync_obj_engine_read_callback(OSyncClientProxy *proxy, void *userdata, OSyncError *error)
 {
 	OSyncSinkEngine *sinkengine = userdata;
 	OSyncObjEngine *engine = sinkengine->engine;
@@ -279,12 +296,10 @@ static void _obj_engine_read_callback(OSyncClientProxy *proxy, void *userdata, O
 		if (osync_bitcount(engine->sink_get_changes) < osync_bitcount(engine->sink_connects)) {
 			osync_error_set(&locerror, OSYNC_ERROR_GENERIC, "Fewer sink_engines reported get_changes than connected");
 			osync_obj_engine_set_error(engine, locerror);
-			osync_obj_engine_event(engine, OSYNC_ENGINE_EVENT_ERROR);
 		} else {
 			/* We are now done reading the changes. so we can now start to create the mappings, conflicts etc */
 			if (!osync_obj_engine_map_changes(engine, &locerror)) {
 				osync_obj_engine_set_error(engine, locerror);
-				osync_obj_engine_event(engine, OSYNC_ENGINE_EVENT_ERROR);
 			} else {
 				GList *m;
 				for (m = engine->mapping_engines; m; m = m->next) {
@@ -292,10 +307,11 @@ static void _obj_engine_read_callback(OSyncClientProxy *proxy, void *userdata, O
 					if (!mapping_engine->synced)
 						osync_mapping_engine_check_conflict(mapping_engine);
 				}
-				
-				osync_obj_engine_event(engine, OSYNC_ENGINE_EVENT_READ);
 			}
+
 		}
+
+		osync_obj_engine_event(engine, OSYNC_ENGINE_EVENT_READ, locerror ? locerror : error);
 	} else
 		osync_trace(TRACE_INTERNAL, "Not yet: %i", osync_bitcount(engine->sink_errors | engine->sink_get_changes));
 	
@@ -349,9 +365,20 @@ osync_bool osync_obj_engine_receive_change(OSyncObjEngine *objengine, OSyncClien
 	return TRUE;
 }
 
-static void _generate_written_event(OSyncObjEngine *engine)
+/* Note: This function got shared between _osync_obj_engine_commit_change_callback() and
+         osync_obj_engine_written_callback(). Those function call _osync_obj_engine_generate_written_event()
+	 with the most recent error and pass it to this function as last argument "error". If no error
+	 appears this function got called with NULL as error.
+
+	 It's quite important that this function only get called with the most recent error. This function
+	 MUST NOT get called with (obj)engine->error.
+
+	 If this functions doesn't get called with the most recent commit/committed_all error OSyncEngine
+	 will get stuck. (testcases: dual_commit_error, dual_commit_timeout, *_commit_*, *_committed_all_*)
+ */
+static void _osync_obj_engine_generate_written_event(OSyncObjEngine *engine, OSyncError *error)
 {
-	osync_trace(TRACE_INTERNAL, "%s: %p", __func__, engine);
+	osync_trace(TRACE_ENTRY, "%s(%p, %p)", __func__, engine, error);
 	/* We need to make sure that all entries are written ... */
 	osync_bool dirty = FALSE;
 	GList *p = NULL;
@@ -376,8 +403,10 @@ static void _generate_written_event(OSyncObjEngine *engine)
 				break;
 			}
 		}
-		if (dirty)
+		if (dirty) {
+			osync_trace(TRACE_EXIT, "%s: Still dirty", __func__);
 			return;
+		}
 	}
 	osync_trace(TRACE_INTERNAL, "%s: Not dirty anymore", __func__);
 
@@ -386,17 +415,25 @@ static void _generate_written_event(OSyncObjEngine *engine)
 		if (osync_bitcount(engine->sink_written) < osync_bitcount(engine->sink_connects)) {
 			osync_error_set(&locerror, OSYNC_ERROR_GENERIC, "Fewer sink_engines reported committed all than connected");
 			osync_obj_engine_set_error(engine, locerror);
-			osync_obj_engine_event(engine, OSYNC_ENGINE_EVENT_ERROR);
-		} else
-			osync_obj_engine_event(engine, OSYNC_ENGINE_EVENT_WRITTEN);
+		} else if (osync_bitcount(engine->sink_errors)) {
+			/* Emit engine-wide error if one of the sinks got an error (tests: single_commit_error, ...) */
+			osync_error_set(&locerror, OSYNC_ERROR_GENERIC, "At least one Sink Engine failed while committing");
+			osync_obj_engine_set_error(engine, locerror);
+		}
+
+		osync_obj_engine_event(engine, OSYNC_ENGINE_EVENT_WRITTEN, locerror ? locerror : error);
+
 	} else
 		osync_trace(TRACE_INTERNAL, "Not yet: %i", osync_bitcount(engine->sink_errors | engine->sink_written));
+
+	osync_trace(TRACE_EXIT, "%s", __func__);
 }
 
-static void _obj_engine_commit_change_callback(OSyncClientProxy *proxy, void *userdata, const char *uid, OSyncError *error)
+static void _osync_obj_engine_commit_change_callback(OSyncClientProxy *proxy, void *userdata, const char *uid, OSyncError *error)
 {
 	OSyncMappingEntryEngine *entry_engine = userdata;
 	OSyncObjEngine *engine = entry_engine->objengine;
+	OSyncSinkEngine *sinkengine = entry_engine->sink_engine;
 	OSyncError *locerror = NULL;
 	
 	osync_trace(TRACE_ENTRY, "%s(%p, %p, %s, %p)", __func__, proxy, userdata, uid, error);
@@ -410,8 +447,15 @@ static void _obj_engine_commit_change_callback(OSyncClientProxy *proxy, void *us
 	long long int id = osync_mapping_entry_get_id(entry);
 	
 	if (error) {
+		/* Error handling (tests: single_commit_error, ...) */
+
+		/* TODO: Review differences between Mapping and Change status events - Are both really needed?! */
 		osync_status_update_change(engine->parent, entry_engine->change, osync_client_proxy_get_member(proxy), entry_engine->mapping_engine->mapping, OSYNC_CHANGE_EVENT_ERROR, error);
-		goto end;
+		osync_status_update_mapping(engine->parent, entry_engine->mapping_engine, OSYNC_MAPPING_EVENT_ERROR, error);
+
+		osync_obj_engine_set_error(engine, error);
+		engine->sink_errors = engine->sink_errors | (0x1 << sinkengine->position);
+		goto error;
 	}
 	
 	if (uid)
@@ -419,8 +463,11 @@ static void _obj_engine_commit_change_callback(OSyncClientProxy *proxy, void *us
 	
 	if (engine->archive) {
 		if (osync_change_get_changetype(entry_engine->change) == OSYNC_CHANGE_TYPE_DELETED) {
+			/* TODO error handling */
 			osync_archive_delete_change(engine->archive, id, objtype, &locerror);
 		} else {
+
+			/* TODO error handling */
 			osync_archive_save_change(engine->archive, id, osync_change_get_uid(entry_engine->change), objtype, osync_mapping_get_id(mapping), osync_member_get_id(member), &locerror);
 		}
 	}
@@ -429,13 +476,15 @@ static void _obj_engine_commit_change_callback(OSyncClientProxy *proxy, void *us
 	osync_status_update_change(engine->parent, entry_engine->change, osync_client_proxy_get_member(proxy), entry_engine->mapping_engine->mapping, OSYNC_CHANGE_EVENT_WRITTEN, NULL);
 	osync_entry_engine_update(entry_engine, NULL);
 	
-end:	
-	_generate_written_event(engine);
-	
 	osync_trace(TRACE_EXIT, "%s", __func__);
+	return;
+
+error:	
+	_osync_obj_engine_generate_written_event(engine, error);
+	osync_trace(TRACE_EXIT_ERROR, "%s: %s", __func__, osync_error_print(&error));
 }
 
-static void _obj_engine_written_callback(OSyncClientProxy *proxy, void *userdata, OSyncError *error)
+static void _osync_obj_engine_written_callback(OSyncClientProxy *proxy, void *userdata, OSyncError *error)
 {
 	OSyncSinkEngine *sinkengine = userdata;
 	OSyncObjEngine *engine = sinkengine->engine;
@@ -451,12 +500,12 @@ static void _obj_engine_written_callback(OSyncClientProxy *proxy, void *userdata
 		osync_status_update_member(engine->parent, osync_client_proxy_get_member(proxy), OSYNC_CLIENT_EVENT_WRITTEN, engine->objtype, NULL);
 	}
 			
-	_generate_written_event(engine);
+	_osync_obj_engine_generate_written_event(engine, error);
 	
 	osync_trace(TRACE_EXIT, "%s", __func__);
 }
 
-static void _obj_engine_sync_done_callback(OSyncClientProxy *proxy, void *userdata, OSyncError *error)
+static void _osync_obj_engine_sync_done_callback(OSyncClientProxy *proxy, void *userdata, OSyncError *error)
 {
 	OSyncSinkEngine *sinkengine = userdata;
 	OSyncObjEngine *engine = sinkengine->engine;
@@ -477,9 +526,9 @@ static void _obj_engine_sync_done_callback(OSyncClientProxy *proxy, void *userda
 		if (osync_bitcount(engine->sink_sync_done) < osync_bitcount(engine->sink_connects)) {
 			osync_error_set(&locerror, OSYNC_ERROR_GENERIC, "Fewer sink_engines reported sync_done than connected");
 			osync_obj_engine_set_error(engine, locerror);
-			osync_obj_engine_event(engine, OSYNC_ENGINE_EVENT_ERROR);
-		} else
-			osync_obj_engine_event(engine, OSYNC_ENGINE_EVENT_SYNC_DONE);
+		}
+
+		osync_obj_engine_event(engine, OSYNC_ENGINE_EVENT_SYNC_DONE, locerror ? locerror : error);
 	} else
 		osync_trace(TRACE_INTERNAL, "Not yet: %i", osync_bitcount(engine->sink_errors | engine->sink_sync_done));
 	
@@ -714,6 +763,8 @@ error:
 
 void osync_obj_engine_finalize(OSyncObjEngine *engine)
 {
+	osync_trace(TRACE_ENTRY, "%s(%p)", __func__, engine);
+
 	engine->slowsync = FALSE;
 	engine->written = FALSE;
 
@@ -741,6 +792,7 @@ void osync_obj_engine_finalize(OSyncObjEngine *engine)
 	if (engine->mapping_table)
 		osync_mapping_table_close(engine->mapping_table);
 
+	osync_trace(TRACE_EXIT, "%s", __func__);
 }
 
 const char *osync_obj_engine_get_objtype(OSyncObjEngine *engine)
@@ -784,8 +836,11 @@ static OSyncObjFormat **_get_member_formats(OSyncFormatEnv *env, OSyncClientProx
 	const OSyncList *f = NULL;
 	int i = 0;
 	for (f = formats; f; f = f->next) {
-		const char *formatstr = f->data;
+		const char **format_vertice = f->data;
+		const char *formatstr = format_vertice[0];
+		const char *format_configstr = format_vertice[1];
 		OSyncObjFormat *format = osync_format_env_find_objformat(env, formatstr);
+		osync_objformat_set_config(format, format_configstr);
 		if (!format) {
 			g_free(formatArray);
 			osync_error_set(error, OSYNC_ERROR_GENERIC, "Unable to find a valid object format for \"%s\"", formatstr);
@@ -811,6 +866,7 @@ osync_bool osync_obj_engine_command(OSyncObjEngine *engine, OSyncEngineCmd cmd,
 	GList *m = NULL;
 	GList *e = NULL;
 	OSyncSinkEngine *sinkengine =  NULL;
+
 	
 	osync_trace(TRACE_ENTRY, "%s(%p, %i, %p)", __func__, engine, cmd, error);
 	osync_assert(engine);
@@ -819,7 +875,8 @@ osync_bool osync_obj_engine_command(OSyncObjEngine *engine, OSyncEngineCmd cmd,
 		case OSYNC_ENGINE_COMMAND_CONNECT:
 			for (p = engine->sink_engines; p; p = p->next) {
 				sinkengine = p->data;
-				if (!osync_client_proxy_connect(sinkengine->proxy, _obj_engine_connect_callback, sinkengine, engine->objtype, engine->slowsync, error))
+
+				if (!osync_client_proxy_connect(sinkengine->proxy, _osync_obj_engine_connect_callback, sinkengine, engine->objtype, engine->slowsync, error))
 					goto error;
 			}
 			break;
@@ -833,7 +890,7 @@ osync_bool osync_obj_engine_command(OSyncObjEngine *engine, OSyncEngineCmd cmd,
 					if (!change)
 						continue;
 
-					if (!osync_client_proxy_read(sinkengine->proxy, _obj_engine_read_ignored_callback, sinkengine, change, error))
+					if (!osync_client_proxy_read(sinkengine->proxy, _osync_obj_engine_read_ignored_callback, sinkengine, change, error))
 						goto error;
 				}
 			}
@@ -856,12 +913,12 @@ osync_bool osync_obj_engine_command(OSyncObjEngine *engine, OSyncEngineCmd cmd,
 
 				/* Is there at least one other writeable sink? */
 				if (objtype_sink && osync_objtype_sink_get_write(objtype_sink) && write_sinks) {
-					_obj_engine_read_callback(sinkengine->proxy, sinkengine, *error);
+					_osync_obj_engine_read_callback(sinkengine->proxy, sinkengine, *error);
 					osync_trace(TRACE_INTERNAL, "no other writable sinks .... SKIP");
 					continue;
 				}
 
-				if (!osync_client_proxy_get_changes(sinkengine->proxy, _obj_engine_read_callback, sinkengine, engine->objtype, engine->slowsync, error))
+				if (!osync_client_proxy_get_changes(sinkengine->proxy, _osync_obj_engine_read_callback, sinkengine, engine->objtype, engine->slowsync, error))
 					goto error;
 			}
 
@@ -941,8 +998,8 @@ osync_bool osync_obj_engine_command(OSyncObjEngine *engine, OSyncEngineCmd cmd,
 						osync_assert(entry_engine->change);
 						OSyncChange *change = entry_engine->change;
 
-						/* Convert to requested target format */
-						if (osync_group_get_converter_enabled(osync_engine_get_group(engine->parent))) {
+						/* Convert to requested target format if the changetype is not DELETED */
+						if (osync_group_get_converter_enabled(osync_engine_get_group(engine->parent)) && (osync_change_get_changetype(change) != OSYNC_CHANGE_TYPE_DELETED)) {
 
 							
 							osync_trace(TRACE_INTERNAL, "Starting to convert from objtype %s and format %s", osync_change_get_objtype(entry_engine->change), osync_objformat_get_name(osync_change_get_objformat(entry_engine->change)));
@@ -956,11 +1013,19 @@ osync_bool osync_obj_engine_command(OSyncObjEngine *engine, OSyncEngineCmd cmd,
 							if (!formats)
 								goto error;
 							
-							OSyncFormatConverterPath *path = osync_format_env_find_path_formats(engine->formatenv, osync_change_get_objformat(entry_engine->change), formats, error);
+							OSyncFormatConverterPath *path = osync_format_env_find_path_formats_with_detectors(engine->formatenv, osync_change_get_data(entry_engine->change), formats, error);
 							if (!path) {
 								g_free(formats);
 								goto error;
 							}
+
+							int length = osync_converter_path_num_edges(path);
+							OSyncFormatConverter *converter = osync_converter_path_nth_edge(path, length - 1);
+							if (converter) {
+								OSyncObjFormat *format = osync_converter_get_targetformat(converter);
+								osync_converter_path_set_config( path, osync_objformat_get_config(format));
+							}
+
 							g_free(formats);
 							
 							if (!osync_format_env_convert(engine->formatenv, path, osync_change_get_data(entry_engine->change), error)) {
@@ -975,9 +1040,9 @@ osync_bool osync_obj_engine_command(OSyncObjEngine *engine, OSyncEngineCmd cmd,
 							g_free(objtype);
 						}
 						
-						osync_trace(TRACE_INTERNAL, "Writing change %s, changetype %i, format %s , objtype %s from member %lli", osync_change_get_uid(change), osync_change_get_changetype(change), osync_objformat_get_name(osync_change_get_objformat(change)), osync_change_get_objtype(change), osync_member_get_id(osync_client_proxy_get_member(sinkengine->proxy)));
+						osync_trace(TRACE_INTERNAL, "Writing change %s, changetype %i, format %s , format conversion config %s , objtype %s from member %lli", osync_change_get_uid(change), osync_change_get_changetype(change), osync_objformat_get_name(osync_change_get_objformat(change)), osync_objformat_get_config(osync_change_get_objformat(change)), osync_change_get_objtype(change), osync_member_get_id(osync_client_proxy_get_member(sinkengine->proxy)));
 	
-						if (!osync_client_proxy_commit_change(sinkengine->proxy, _obj_engine_commit_change_callback, entry_engine, osync_entry_engine_get_change(entry_engine), error))
+						if (!osync_client_proxy_commit_change(sinkengine->proxy, _osync_obj_engine_commit_change_callback, entry_engine, osync_entry_engine_get_change(entry_engine), error))
 							goto error;
 					} else if (entry_engine->change) {
 						OSyncMapping *mapping = entry_engine->mapping_engine->mapping;
@@ -997,23 +1062,42 @@ osync_bool osync_obj_engine_command(OSyncObjEngine *engine, OSyncEngineCmd cmd,
 					}
 				}
 				
-				if (!osync_client_proxy_committed_all(sinkengine->proxy, _obj_engine_written_callback, sinkengine, engine->objtype, error))
+				if (!osync_client_proxy_committed_all(sinkengine->proxy, _osync_obj_engine_written_callback, sinkengine, engine->objtype, error))
 					goto error;
 			}
 			break;
 		case OSYNC_ENGINE_COMMAND_SYNC_DONE:
 			for (p = engine->sink_engines; p; p = p->next) {
 				sinkengine = p->data;
-				if (!osync_client_proxy_sync_done(sinkengine->proxy, _obj_engine_sync_done_callback, sinkengine, engine->objtype, error))
+				if (!osync_client_proxy_sync_done(sinkengine->proxy, _osync_obj_engine_sync_done_callback, sinkengine, engine->objtype, error))
 					goto error;
 			}
 			break;
-		case OSYNC_ENGINE_COMMAND_DISCONNECT:
+		case OSYNC_ENGINE_COMMAND_DISCONNECT:;
+
+			osync_bool proxy_disconnect = FALSE;
+
 			for (p = engine->sink_engines; p; p = p->next) {
 				sinkengine = p->data;
-				if (!osync_client_proxy_disconnect(sinkengine->proxy, _obj_engine_disconnect_callback, sinkengine, engine->objtype, error))
+
+				/* Don't call client disconnect functions if the sink is already disconnected.
+				   This avoids unintended disconnect calls of clients/plugins which might not prepared
+				   for a disconnect call when their never got connected. (testcases: *_connect_error, *_connect_timeout ..) */
+				if (!osync_sink_engine_is_connected(sinkengine))
+					continue;
+
+				proxy_disconnect = TRUE;
+
+				if (!osync_client_proxy_disconnect(sinkengine->proxy, _osync_obj_engine_disconnect_callback, sinkengine, engine->objtype, error))
 					goto error;
 			}
+			
+			/* If no client needs to be disconnected, we MUST NOT expected any 
+			   disconnected_callback which generates an OSYNC_ENGINE_EVENT_DISCONNECTED event.
+			   So we directly generate such event on our own. (testcases: double_connect_*, triple_connnect_*) */ 
+			if (!proxy_disconnect)
+				_osync_obj_engine_generate_event_disconnected(engine, NULL);
+
 			break;
 		case OSYNC_ENGINE_COMMAND_SOLVE:
 		case OSYNC_ENGINE_COMMAND_DISCOVER:
@@ -1025,18 +1109,37 @@ osync_bool osync_obj_engine_command(OSyncObjEngine *engine, OSyncEngineCmd cmd,
 
 error:
 	osync_trace(TRACE_EXIT_ERROR, "%s: %s", __func__, osync_error_print(error));
-	osync_error_unref(error);
 	return FALSE;
 }
 
 
-void osync_obj_engine_event(OSyncObjEngine *engine, OSyncEngineEvent event)
+void osync_obj_engine_event(OSyncObjEngine *engine, OSyncEngineEvent event, OSyncError *error)
 {
-	osync_trace(TRACE_ENTRY, "%s(%p, %i)", __func__, engine, event);
+	osync_trace(TRACE_ENTRY, "%s(%p, %i, %p)", __func__, engine, event, error);
 	osync_assert(engine);
+
+	/* TODO: Create own enum OSyncObjEngine for objengine events. */
+	osync_assert_msg(event != OSYNC_ENGINE_EVENT_ERROR, "OSyncObjEngine isn't supposed to emit OSYNC_ENGINE_EVENT_ERROR events!");
 	
-	engine->callback(engine, event, engine->error, engine->callback_userdata);
-	
+	/* engine event callback gets called with most recent OSyncError or NULL.
+	   Don't use engine->error for the engine event callback. Previous appears errors
+	   in this objengine would get passed to this engine, which will be interpeted by the
+	   engine as error for the current event.
+
+	   Example:
+	   
+	   EVENT_CONNECTED		(obj)engine->error: NULL
+	   EVENT_READ **ERROR**		(obj)engine->error: 0x....
+	   # OSyncEngine aborts sync and emit disconnect event, we reply with:
+	   EVENT_DISCONNECTED		(obj)engine->error: 0x.....
+
+	   If we would pass in this case enigne->error instead of the most recent
+	   OSyncError, OSyncEngien would interpret this as the disconnect failed as well.
+	   So we just pass the most recent OSyncError pointer, which could be NULL -> no error.
+	*/
+	     
+	engine->callback(engine, event, error, engine->callback_userdata);
+
 	osync_trace(TRACE_EXIT, "%s", __func__);
 	return;
 }
