@@ -154,7 +154,6 @@ static osync_bool _osync_group_load_members(OSyncGroup *group, const char *path,
 error_free_member:
 	osync_member_unref(member);
 error_close:
-	g_free(filename);
 	g_dir_close(dir);
 error:
 	osync_trace(TRACE_EXIT_ERROR, "%s: %p", __func__, osync_error_print(error));
@@ -290,8 +289,6 @@ void osync_group_unref(OSyncGroup *group)
 	osync_assert(group);
 		
 	if (g_atomic_int_dec_and_test(&(group->ref_count))) {
-		if (group->lock_fd)
-			osync_group_unlock(group);
 		
 		while (group->members)
 			osync_group_remove_member(group, group->members->data);
