@@ -29,13 +29,130 @@
 
 
 /**
- * @ingroup OSEngineQueue
+ * @ingroup OSyncQueue
  * @brief A Queue used for asynchronous communication between thread
  * 
  */
 
 /*@{*/
 
+static
+gboolean _timeout_prepare(GSource *source, gint *timeout_)
+{
+	/* TODO adapt *timeout_ value to shortest message timeout value...
+	GTimeVal current_time;
+	g_source_get_current_time(source, &current_time);
+	*/
+
+	*timeout_ = 1;
+	return FALSE;
+}
+
+static
+gboolean _timeout_check(GSource *source)
+{
+	GList *p;
+	GTimeVal current_time;
+	OSyncTimeoutInfo *toinfo;
+	OSyncPendingMessage *pending;
+
+	OSyncQueue *queue = *((OSyncQueue **)(source + 1));
+
+	g_source_get_current_time(source, &current_time);
+
+	/* Search for the pending reply. We have to lock the
+	 * list since another thread might be duing the updates */
+	g_mutex_lock(queue->pendingLock);
+
+	for (p = queue->pendingReplies; p; p = p->next) {
+		pending = p->data;
+
+		if (!pending->timeout_info)
+			continue;
+
+		toinfo = pending->timeout_info;
+
+		if (current_time.tv_sec >= toinfo->expiration.tv_sec 
+			|| (current_time.tv_sec == toinfo->expiration.tv_sec 
+			&& current_time.tv_usec >= toinfo->expiration.tv_usec)) {
+			/* Unlock the pending lock since the messages might be sent during the callback */
+			g_mutex_unlock(queue->pendingLock);
+
+			return TRUE;
+		}
+	}
+	/* Unlock the pending lock since the messages might be sent during the callback */
+	g_mutex_unlock(queue->pendingLock);
+
+	return FALSE;
+}
+
+static
+gboolean _timeout_dispatch(GSource *source, GSourceFunc callback, gpointer user_data)
+{
+	GList *p;
+	OSyncPendingMessage *pending;
+	OSyncTimeoutInfo *toinfo;
+
+	osync_trace(TRACE_INTERNAL, "%s(%p)", __func__, user_data);
+
+	OSyncQueue *queue = *((OSyncQueue **)(source + 1));
+
+	GTimeVal current_time;
+	g_source_get_current_time (source, &current_time);
+
+	/* Search for the pending reply. We have to lock the
+	 * list since another thread might be duing the updates */
+	g_mutex_lock(queue->pendingLock);
+
+	for (p = queue->pendingReplies; p; p = p->next) {
+		pending = p->data;
+
+		if (!pending->timeout_info)
+			continue;
+
+		toinfo = pending->timeout_info;
+
+		if (current_time.tv_sec == toinfo->expiration.tv_sec
+				|| current_time.tv_sec >= toinfo->expiration.tv_sec && current_time.tv_usec >= toinfo->expiration.tv_usec) {
+
+			/* Call the callback of the pending message */
+			osync_assert(pending->callback);
+			OSyncError *error = NULL;
+			OSyncError *timeouterr = NULL;
+			osync_error_set(&timeouterr, OSYNC_ERROR_IO_ERROR, "Timeout.");
+			OSyncMessage *errormsg = osync_message_new_errorreply(NULL, timeouterr, &error);
+			osync_error_unref(&timeouterr);
+
+			/* Remove first the pending message!
+			   To avoid that _incoming_dispatch catchs this message
+			   when we're releasing the lock. If _incoming_dispatch
+			   would catch this message, the pending callback
+			   gets called twice! */
+
+			queue->pendingReplies = g_list_remove(queue->pendingReplies, pending);
+			/* Unlock the pending lock since the messages might be sent during the callback */
+			g_mutex_unlock(queue->pendingLock);
+
+			pending->callback(errormsg, pending->user_data);
+			osync_message_unref(errormsg);
+
+			// TODO: Refcounting for OSyncPendingMessage
+			g_free(pending->timeout_info);
+			g_free(pending);
+
+			/* Lock again, to keep the iteration of the pendingReplies list atomic. */
+			g_mutex_lock(queue->pendingLock);
+
+			break;
+		}
+	}
+	
+	g_mutex_unlock(queue->pendingLock);
+
+	return TRUE;
+}
+
 static
 gboolean _incoming_prepare(GSource *source, gint *timeout_)
 {
@@ -79,19 +196,29 @@ gboolean _incoming_dispatch(GSource *source, GSourceFunc callback, gpointer user
 				pending = p->data;
 			
 				if (pending->id == osync_message_get_id(message)) {
+
+					/* Remove first the pending message!
+					   To avoid that _timeout_dispatch catchs this message
+					   when we're releasing the lock. If _timeout_dispatch
+					   would catch this message, the pending callback
+					   gets called twice! */
+
+					queue->pendingReplies = g_list_remove(queue->pendingReplies, pending);
+
 					/* Unlock the pending lock since the messages might be sent during the callback */
 					g_mutex_unlock(queue->pendingLock);
 			
 					/* Call the callback of the pending message */
 					osync_assert(pending->callback);
 					pending->callback(message, pending->user_data);
-					
-					/* Then remove the pending message and free it.
-					 * But first, lock the queue to make sure that the removal
-					 * is atomic */
-					g_mutex_lock(queue->pendingLock);
-					queue->pendingReplies = g_list_remove(queue->pendingReplies, pending);
+
+					// TODO: Refcounting for OSyncPendingMessage
+					if (pending->timeout_info)
+						g_free(pending->timeout_info);
 					g_free(pending);
+
+					/* Lock again, to keep the iteration of the pendingReplies list atomic. */
+					g_mutex_lock(queue->pendingLock);
 					break;
 				}
 			}
@@ -581,8 +708,14 @@ void osync_queue_free(OSyncQueue *queue)
 
 	while (queue->pendingReplies) {
 		pending = queue->pendingReplies->data;
-		g_free(pending);
+
 		queue->pendingReplies = g_list_remove(queue->pendingReplies, pending);
+
+		// TODO: Refcounting for OSyncPendingMessage
+		if (pending->timeout_info)
+			g_free(pending->timeout_info);
+
+		g_free(pending);
 	}
 
 	if (queue->name)
@@ -704,6 +837,20 @@ osync_bool osync_queue_connect(OSyncQueue *queue, OSyncQueueType type, OSyncErro
 	g_source_attach(queue->read_source, queue->context);
 	if (queue->context)
 		g_main_context_ref(queue->context);
+
+	queue->timeout_functions = g_malloc0(sizeof(GSourceFuncs));
+	queue->timeout_functions->prepare = _timeout_prepare;
+	queue->timeout_functions->check = _timeout_check;
+	queue->timeout_functions->dispatch = _timeout_dispatch;
+	queue->timeout_functions->finalize = NULL;
+
+	queue->timeout_source = g_source_new(queue->timeout_functions, sizeof(GSource) + sizeof(OSyncQueue *));
+	queueptr = (OSyncQueue **)(queue->timeout_source + 1);
+	*queueptr = queue;
+	g_source_set_callback(queue->timeout_source, NULL, queue, NULL);
+	g_source_attach(queue->timeout_source, queue->context);
+	if (queue->context)
+		g_main_context_ref(queue->context);
 	
 	osync_thread_start(queue->thread);
 	
@@ -739,6 +886,11 @@ osync_bool osync_queue_disconnect(OSyncQueue *queue, OSyncError **error)
 		
 	//g_source_unref(queue->read_source);
 	
+	if (queue->timeout_functions) {
+		g_free(queue->timeout_functions);
+		queue->timeout_functions = NULL;
+	}
+		
 	_osync_queue_stop_incoming(queue);
 	
 	/* We have to empty the incoming queue if we disconnect the queue. Otherwise, the
@@ -882,13 +1034,9 @@ OSyncMessage *osync_queue_get_message(OSyncQueue *queue)
  * the lower 2 bytes are a random number
  * 
  * */
-static long long int gen_id()
+static long long int gen_id(const GTimeVal *tv)
 {
-    GTimeVal tv;
-
-    g_get_current_time(&tv);
-
-    long long int now = (tv.tv_sec * 1000000 + tv.tv_usec) << 16;
+    long long int now = (tv->tv_sec * 1000000 + tv->tv_usec) << 16;
     long long int rnd = ((long long int)g_random_int()) & 0xFFFF;
     
     return now | rnd;
@@ -896,18 +1044,41 @@ static long long int gen_id()
 
 osync_bool osync_queue_send_message(OSyncQueue *queue, OSyncQueue *replyqueue, OSyncMessage *message, OSyncError **error)
 {
-	osync_trace(TRACE_ENTRY, "%s(%p, %p, %p, %p)", __func__, queue, replyqueue, message, error);
-	
+	return osync_queue_send_message_with_timeout(queue, replyqueue, message, 0, error);
+}
+
+osync_bool osync_queue_send_message_with_timeout(OSyncQueue *queue, OSyncQueue *replyqueue, OSyncMessage *message, unsigned int timeout, OSyncError **error)
+{
+	osync_trace(TRACE_ENTRY, "%s(%p, %p, %p, %u, %p)", __func__, queue, replyqueue, message, timeout, error);
+
 	if (osync_message_get_handler(message)) {
 		osync_assert(replyqueue);
 		OSyncPendingMessage *pending = osync_try_malloc0(sizeof(OSyncPendingMessage), error);
 		if (!pending)
 			goto error;
 		
-		long long int id = gen_id();
+		GTimeVal current_time;
+
+		/* g_sourcet_get_current_time used cached time ... hopefully faster then g_get_.._time() */
+		g_source_get_current_time(queue->timeout_source, &current_time);
+
+		long long int id = gen_id(&current_time);
 		osync_message_set_id(message, id);
 		pending->id = id;
 		osync_trace(TRACE_INTERNAL, "Setting id %lli for pending reply", id);
+
+		if (timeout) {
+			OSyncTimeoutInfo *toinfo = osync_try_malloc0(sizeof(OSyncTimeoutInfo), error);
+			if (!toinfo)
+				goto error;
+
+			toinfo->expiration = current_time;
+			toinfo->expiration.tv_sec += timeout;
+
+			pending->timeout_info = toinfo;
+		} else {
+			osync_trace(TRACE_INTERNAL, "handler message got sent without timeout!: %s", osync_message_get_commandstr(message));
+		}
 		
 		pending->callback = osync_message_get_handler(message);
 		pending->user_data = osync_message_get_handler_data(message);
@@ -930,18 +1101,6 @@ error:
 	return FALSE;
 }
 
-osync_bool osync_queue_send_message_with_timeout(OSyncQueue *queue, OSyncQueue *replyqueue, OSyncMessage *message, int timeout, OSyncError **error)
-{
-	osync_trace(TRACE_ENTRY, "%s(%p, %p, %p)", __func__, queue, message, error);
-	
-	/*TODO: add timeout handling */
-	
-	osync_bool ret = osync_queue_send_message(queue, replyqueue, message, error);
-	
-	osync_trace(ret ? TRACE_EXIT : TRACE_EXIT_ERROR, "%s: %s", __func__, osync_error_print(error));
-	return ret;
-}
-
 osync_bool osync_queue_is_alive(OSyncQueue *queue)
 {
 	osync_trace(TRACE_ENTRY, "%s(%p)", __func__, queue);
