@@ -45,6 +45,7 @@ osync_bool osync_marshal_data(OSyncMessage *message, OSyncData *data, OSyncError
 	
 	/* Write the format and objtype first */
 	osync_message_write_string(message, osync_objformat_get_name(objformat));
+	osync_message_write_string(message, osync_objformat_get_config(objformat));
 	osync_message_write_string(message, osync_data_get_objtype(data));
 
 	/* Now we get the pointer to the data */
@@ -89,8 +90,10 @@ osync_bool osync_demarshal_data(OSyncMessage *message, OSyncData **data, OSyncFo
 	
 	/* Get the objtype and format */
 	char *objformat = NULL;
+	char *objformat_config = NULL;
 	char *objtype = NULL;
 	osync_message_read_string(message, &objformat);
+	osync_message_read_string(message, &objformat_config);
 	osync_message_read_string(message, &objtype);
 	
 	/* Search for the format */
@@ -99,7 +102,8 @@ osync_bool osync_demarshal_data(OSyncMessage *message, OSyncData **data, OSyncFo
 		osync_error_set(error, OSYNC_ERROR_GENERIC, "Unable to find objformat %s", objformat);
 		goto error;
 	}
-	
+	osync_objformat_set_config(format, objformat_config);
+
 	unsigned int input_size = 0;
 	char *input_data = NULL;
 	
@@ -217,7 +221,18 @@ osync_bool osync_marshal_objtype_sink(OSyncMessage *message, OSyncObjTypeSink *s
 	 * write function (bool)
 	 * number of formats
 	 * format list (string)
-	 * enabled */
+	 * enabled (int)
+	 * timeout connect (int)
+	 * timeout disconnect (int)
+	 * timeout get_changes (int)
+	 * timeout commit (int)
+	 * timeout batch_commit (int)
+	 * timeout committed_all (int)
+	 * timeout sync_done (int)
+	 * timeout read (int)
+	 * timeout write (int)
+	 * 
+	 */
 	
 	int i = 0;
 	int num = osync_objtype_sink_num_objformats(sink);
@@ -230,10 +245,27 @@ osync_bool osync_marshal_objtype_sink(OSyncMessage *message, OSyncObjTypeSink *s
 	osync_message_write_int(message, num);
 	for (i = 0; i < num; i++) {
 		const char *format = osync_objtype_sink_nth_objformat(sink, i);
+		const char *format_config = osync_objtype_sink_nth_objformat_config(sink, i);
 		osync_message_write_string(message, format);
+		osync_message_write_string(message, format_config);
 	}
 	
+	/* enabled */
 	osync_message_write_int(message, osync_objtype_sink_is_enabled(sink));
+
+	/* timeouts */
+	osync_message_write_int(message, osync_objtype_sink_get_connect_timeout(sink));
+	osync_message_write_int(message, osync_objtype_sink_get_disconnect_timeout(sink));
+
+	osync_message_write_int(message, osync_objtype_sink_get_getchanges_timeout(sink));
+	osync_message_write_int(message, osync_objtype_sink_get_commit_timeout(sink));
+	osync_message_write_int(message, osync_objtype_sink_get_batchcommit_timeout(sink));
+	osync_message_write_int(message, osync_objtype_sink_get_committedall_timeout(sink));
+	osync_message_write_int(message, osync_objtype_sink_get_syncdone_timeout(sink));
+
+	osync_message_write_int(message, osync_objtype_sink_get_read_timeout(sink));
+	osync_message_write_int(message, osync_objtype_sink_get_write_timeout(sink));
+
 	
 	return TRUE;
 }
@@ -248,7 +280,18 @@ osync_bool osync_demarshal_objtype_sink(OSyncMessage *message, OSyncObjTypeSink
 	 * write function (bool)
 	 * number of formats
 	 * format list (string)
-	 * enabled */
+	 * enabled (int)
+	 * timeout connect (int)
+	 * timeout disconnect (int)
+	 * timeout get_changes (int)
+	 * timeout commit (int)
+	 * timeout batch_commit (int)
+	 * timeout committed_all (int)
+	 * timeout sync_done (int)
+	 * timeout read (int)
+	 * timeout write (int)
+	 * 
+	 */
 	
 	*sink = osync_objtype_sink_new(NULL, error);
 	if (!*sink)
@@ -256,9 +299,10 @@ osync_bool osync_demarshal_objtype_sink(OSyncMessage *message, OSyncObjTypeSink
 
 	char *name = NULL;
 	int num_formats = 0;
-	int enabled = 0;
+	int enabled = 0, timeout = 0;
 	int read = 0, get_changes = 0, write = 0;
 	char *format = NULL;
+	char *format_config = NULL;
 	
  	osync_message_read_string(message, &name);
  	osync_objtype_sink_set_name(*sink, name);
@@ -277,13 +321,44 @@ osync_bool osync_demarshal_objtype_sink(OSyncMessage *message, OSyncObjTypeSink
 	int i = 0;
 	for (i = 0; i < num_formats; i++) {
  		osync_message_read_string(message, &format);
-		osync_objtype_sink_add_objformat(*sink, format);
+ 		osync_message_read_string(message, &format_config);
+		osync_objtype_sink_add_objformat_with_config(*sink, format, format_config);
  		g_free(format);
+ 		g_free(format_config);
 	}
 
+	/* enabled */
 	osync_message_read_int(message, &enabled);
 	osync_objtype_sink_set_enabled(*sink, enabled);
 
+	/* timeouts */
+	osync_message_read_int(message, &timeout);
+	osync_objtype_sink_set_connect_timeout(*sink, timeout);
+
+	osync_message_read_int(message, &timeout);
+	osync_objtype_sink_set_disconnect_timeout(*sink, timeout);
+
+	osync_message_read_int(message, &timeout);
+	osync_objtype_sink_set_getchanges_timeout(*sink, timeout);
+
+	osync_message_read_int(message, &timeout);
+	osync_objtype_sink_set_commit_timeout(*sink, timeout);
+
+	osync_message_read_int(message, &timeout);
+	osync_objtype_sink_set_batchcommit_timeout(*sink, timeout);
+
+	osync_message_read_int(message, &timeout);
+	osync_objtype_sink_set_committedall_timeout(*sink, timeout);
+
+	osync_message_read_int(message, &timeout);
+	osync_objtype_sink_set_syncdone_timeout(*sink, timeout);
+
+	osync_message_read_int(message, &timeout);
+	osync_objtype_sink_set_read_timeout(*sink, timeout);
+
+	osync_message_read_int(message, &timeout);
+	osync_objtype_sink_set_write_timeout(*sink, timeout);
+
 	return TRUE;
 
 error:
