@@ -23,9 +23,6 @@
 #ifndef OPENSYNCMERGER_INTERNALS_H_
 #define OPENSYNCMERGER_INTERNALS_H_
 
-#include <sqlite3.h>
-#include <libxml/parser.h>
-#include <libxml/xmlschemas.h>
 #include <opensync/opensync_xml.h>
 
 #include "opensync_capabilities_internals.h"
