@@ -279,7 +279,7 @@ const char *osync_xmlfield_get_attr(OSyncXMLField *xmlfield, const char *attr)
     prop = xmlHasProp(xmlfield->node, BAD_CAST attr);
     if(prop == NULL)
     	return NULL;
-	return (const char *)osxml_attr_get_content(prop);
+	return (const char *)osync_xml_attr_get_content(prop);
 //	return (const char *)prop->children->content;
 //	return (const char *)xmlGetProp(xmlfield->node, BAD_CAST attr);
 }
@@ -352,7 +352,7 @@ const char *osync_xmlfield_get_nth_attr_value(OSyncXMLField *xmlfield, int nth)
 	
 	for(count=0; attr != NULL; count++) {
 		if(count == nth)
-			return (const char *)osxml_attr_get_content(attr);
+			return (const char *)osync_xml_attr_get_content(attr);
 //			return (const char *)attr->children->content;
 //			return (const char *)xmlNodeGetContent((xmlNodePtr)attr);
 		attr = attr->next;
@@ -374,7 +374,7 @@ const char *osync_xmlfield_get_key_value(OSyncXMLField *xmlfield, const char *ke
 	xmlNodePtr cur = xmlfield->node->children;
 	for(; cur != NULL; cur = cur->next) {
 		if(!xmlStrcmp(cur->name, BAD_CAST key))
-				return (const char *)osxml_node_get_content(cur);
+				return (const char *)osync_xml_node_get_content(cur);
 //				return (const char *)cur->children->content;
 //				return (const char *)xmlNodeGetContent(cur);
 	}
@@ -480,7 +480,7 @@ const char *osync_xmlfield_get_nth_key_value(OSyncXMLField *xmlfield, int nth)
 	
 	for(count=0; child != NULL; count++) {
 		if(count == nth)
-			return (const char *)osxml_node_get_content(child);
+			return (const char *)osync_xml_node_get_content(child);
 //			return (const char *)child->children->content;
 //			return (const char *)xmlNodeGetContent(child);
 		child = child->next;
@@ -570,6 +570,7 @@ osync_bool osync_xmlfield_compare(OSyncXMLField *xmlfield1, OSyncXMLField *xmlfi
 		do{
 			/* both lists must have the same length */	
 			if(g_slist_length(keylist1) != g_slist_length(keylist2)) {
+				osync_trace(TRACE_INTERNAL, "It's not the same anymore...");
 				same = FALSE;
 				break;
 			}
@@ -582,7 +583,7 @@ osync_bool osync_xmlfield_compare(OSyncXMLField *xmlfield1, OSyncXMLField *xmlfi
 				cur_list2 = keylist2;
 	
 				do {
-					if(!xmlStrcmp(osxml_node_get_content(cur_list1->data), osxml_node_get_content(cur_list2->data)))
+					if(!xmlStrcmp(osync_xml_node_get_content(cur_list1->data), osync_xml_node_get_content(cur_list2->data)))
 						break;
 					cur_list2 = g_slist_next(cur_list2);
 					if(cur_list2 == NULL) {
@@ -685,8 +686,8 @@ osync_bool osync_xmlfield_compare_similar(OSyncXMLField *xmlfield1, OSyncXMLFiel
 				break;	
 			}
 			
-			while(xmlStrcmp(osxml_node_get_content((xmlNodePtr)cur_list1->data),
-							 osxml_node_get_content((xmlNodePtr)cur_list2->data))) {		
+			while(xmlStrcmp(osync_xml_node_get_content((xmlNodePtr)cur_list1->data),
+							 osync_xml_node_get_content((xmlNodePtr)cur_list2->data))) {		
 //			while(strcmp((const char *)xmlNodeGetContent(cur_list1->data),
 //						 (const char *)xmlNodeGetContent(cur_list2->data)) != 0) {
 				cur_list2 = g_slist_next(cur_list2);
