@@ -50,10 +50,69 @@ int _osync_xmlformat_get_points(OSyncXMLPoints points[], int* cur_pos, int basic
 			return points[(*cur_pos)].points;
 		if(res > 0)
 			return basic_points;
-	};
+	}
+
 	return basic_points;
 }
 
+/**
+ * @brief Search for the points in the sorted OSyncXMLPoints array for the fieldname of xmlfield and handle ignored fields.
+ * 
+ *  This uses _osync_xmlformat_get_points() but handle the case if the field needs to be ignored.
+ *  Ignored fields doesn't have influence on the compare result. This is needed to keep the compare 
+ *  result SAME if an ignored fields are the only differences between the entries.
+ *
+ *  0 Points got returned for ignored fields to avoid influence of the collected_points value and let
+ *  collected_points not raise over the threshold value. This could change the compare result to SIMILAR,
+ *  even if the entry should  MISMATCH, this should be avoided. This happens when the number of ignored fiels
+ *  is the greather equal as the thresold value.
+ *
+ * @param xmlfield Pointer to xmlfield
+ * @param points The sorted points array
+ * @param cur_pos Pointer to the actual line in the points array. Gets incremented if the xmlfield requires this! 
+ * @param basic_points Points which should be returned if fieldname will not found in the points array 
+ * @param same Pointer ot the compare result flag SAME, which got not set to FALSE if the fields should be ignored.
+ * @returns The points for the fieldname. If the field should be ignored 0 points got retunred.
+ */
+static int _osync_xmlformat_subtract_points(OSyncXMLField *xmlfield, OSyncXMLPoints points[], int *cur_pos, int basic_points, int *same) {
+	osync_trace(TRACE_ENTRY, "%s(%p, %p, %p, %i, %p)", __func__, xmlfield, points, cur_pos, basic_points, same);
+	int p = _osync_xmlformat_get_points(points, cur_pos, basic_points, (const char *) xmlfield->node->name);
+
+	/* Stay with SAME as compare result and don't substract any points - if fields should be ignored */
+	if (p != -1) {
+		osync_trace(TRACE_INTERNAL, "Not same anymore - \"%s\" field differs!", xmlfield->node->name);
+		*same = FALSE;
+	} else {
+		osync_trace(TRACE_INTERNAL, "Ignored field: %s", xmlfield->node->name);
+		p = 0;
+	}
+
+	osync_trace(TRACE_EXIT, "%s: %i", __func__, p);
+	return p;
+}
+
+/**
+ * @brief Validate the xmlformat against its schema in inidivual path
+ * @param xmlformat The pointer to a xmlformat object 
+ * @param path The individual schema path. If NULL the default OPENSYNC_SCHEMASDIR is used.
+ * @return TRUE if xmlformat valid else FALSE
+ */
+osync_bool _osync_xmlformat_validate(OSyncXMLFormat *xmlformat, const char *path)
+{
+	osync_assert(xmlformat);
+	
+	char *schemafilepath = g_strdup_printf("%s%c%s%s%s",
+		path ? path : OPENSYNC_SCHEMASDIR,
+		G_DIR_SEPARATOR,
+		"xmlformat-",
+		osync_xmlformat_get_objtype(xmlformat),
+		".xsd");
+ 	osync_bool res = osync_xml_validate_document(xmlformat->doc, schemafilepath);
+ 	g_free(schemafilepath);
+	
+	return res;
+}
+
 /*@}*/
 
 /**
@@ -146,11 +205,13 @@ OSyncXMLFormat *osync_xmlformat_parse(const char *buffer, unsigned int size, OSy
  * @brief Increments the reference counter
  * @param xmlformat The pointer to a xmlformat object
  */
-void osync_xmlformat_ref(OSyncXMLFormat *xmlformat)
+OSyncXMLFormat *osync_xmlformat_ref(OSyncXMLFormat *xmlformat)
 {
 	osync_assert(xmlformat);
 	
 	g_atomic_int_inc(&(xmlformat->ref_count));
+
+	return xmlformat;
 }
 
 /**
@@ -338,17 +399,8 @@ osync_bool osync_xmlformat_assemble(OSyncXMLFormat *xmlformat, char **buffer, un
 osync_bool osync_xmlformat_validate(OSyncXMLFormat *xmlformat)
 {
 	osync_assert(xmlformat);
-	
-	char *schemafilepath = g_strdup_printf("%s%c%s%s%s",
-		OPENSYNC_SCHEMASDIR,
-		G_DIR_SEPARATOR,
-		"xmlformat-",
-		osync_xmlformat_get_objtype(xmlformat),
-		".xsd");
- 	osync_bool res = osxml_validate_document(xmlformat->doc, schemafilepath);
- 	g_free(schemafilepath);
-	
-	return res;
+
+	return _osync_xmlformat_validate(xmlformat, NULL);
 }
 
 /**
@@ -412,53 +464,52 @@ end:
  * @param xmlformat2 The pointer to a xmlformat object
  * @param points The sorted points array
  * @param basic_points Points which should be used if a xmlfield name is not found in the points array
- * @param treshold If the two xmlformats are not the same, then this value will decide if the two xmlformats are similar 
+ * @param threshold If the two xmlformats are not the same, then this value will decide if the two xmlformats are similar 
  * @return One of the values of the OSyncConvCmpResult enumeration
  */
-OSyncConvCmpResult osync_xmlformat_compare(OSyncXMLFormat *xmlformat1, OSyncXMLFormat *xmlformat2, OSyncXMLPoints points[], int basic_points, int treshold)
+OSyncConvCmpResult osync_xmlformat_compare(OSyncXMLFormat *xmlformat1, OSyncXMLFormat *xmlformat2, OSyncXMLPoints points[], int basic_points, int threshold)
 {
-	osync_trace(TRACE_ENTRY, "%s(%p, %p, %p, %i, %i)", __func__, xmlformat1, xmlformat2, points, basic_points, treshold);
+	osync_trace(TRACE_ENTRY, "%s(%p, %p, %p, %i, %i)", __func__, xmlformat1, xmlformat2, points, basic_points, threshold);
 
 	int res, collected_points, cur_pos;
 	OSyncXMLField *xmlfield1 = osync_xmlformat_get_first_field(xmlformat1);
 	OSyncXMLField *xmlfield2 = osync_xmlformat_get_first_field(xmlformat2);
+
 	cur_pos = 0;
 	collected_points = 0;
+
 	osync_bool same = TRUE;
 	
 	while(xmlfield1 != NULL || xmlfield2 != NULL)
 	{
+
 		/* subtract points for xmlfield2*/
 		if(xmlfield1 == NULL) {
-			same = FALSE;
-			collected_points -= _osync_xmlformat_get_points(points, &cur_pos, basic_points, (const char *) xmlfield2->node->name);
+			collected_points -= _osync_xmlformat_subtract_points(xmlfield2, points, &cur_pos, basic_points, &same);
 			xmlfield2 = xmlfield2->next;
 			continue;	
 		}
 		
 		/* subtract points for xmlfield1*/
 		if(xmlfield2 == NULL) {
-			same = FALSE;
-			collected_points -= _osync_xmlformat_get_points(points, &cur_pos, basic_points, (const char *) xmlfield1->node->name);
+			collected_points -= _osync_xmlformat_subtract_points(xmlfield1, points, &cur_pos, basic_points, &same);
 			xmlfield1 = xmlfield1->next;
 			continue;
 		}
-		
+	
 		res = strcmp((const char *)xmlfield1->node->name, (const char *)xmlfield2->node->name);
 		osync_trace(TRACE_INTERNAL, "result of strcmp(): %i (%s || %s)", res, xmlfield1->node->name, xmlfield2->node->name);
 		
 		/* subtract points for xmlfield1*/
 		if(res < 0) {
-			same = FALSE;
-			collected_points -= _osync_xmlformat_get_points(points, &cur_pos, basic_points, (const char *) xmlfield1->node->name);
+			collected_points -= _osync_xmlformat_subtract_points(xmlfield1, points, &cur_pos, basic_points, &same);
 			xmlfield1 = xmlfield1->next;
 			continue;		
 		}
 		
 		/* subtract points for xmlfield2*/
 		if(res > 0) {
-			same = FALSE;
-			collected_points -= _osync_xmlformat_get_points(points, &cur_pos, basic_points, (const char *) xmlfield2->node->name);			
+			collected_points -= _osync_xmlformat_subtract_points(xmlfield2, points, &cur_pos, basic_points, &same);
 			xmlfield2 = xmlfield2->next;
 			continue;			
 		}
@@ -466,12 +517,21 @@ OSyncConvCmpResult osync_xmlformat_compare(OSyncXMLFormat *xmlformat1, OSyncXMLF
 		/* make lists and compare */
 		if(res == 0)
 		{
-			GSList *fieldlist1;
-			GSList *fieldlist2;
-			fieldlist1 = NULL;
-			fieldlist2 = NULL;
-			
+			GSList *fieldlist1 = NULL;
+			GSList *fieldlist2 = NULL;
+
 			const char *curfieldname = (const char *)xmlfield1->node->name;
+
+			/* get the points*/
+			int p = _osync_xmlformat_get_points(points, &cur_pos, basic_points, curfieldname);
+	
+			/* don't compare both fields if they should be ignore to avoid influence of the compare result */
+			if (p == -1) {
+				xmlfield1 = xmlfield1->next;
+				xmlfield2 = xmlfield2->next;
+				continue;
+			}
+
 			do {
 				fieldlist1 = g_slist_prepend(fieldlist1, xmlfield1);
 				xmlfield1 = xmlfield1->next;
@@ -489,9 +549,7 @@ OSyncConvCmpResult osync_xmlformat_compare(OSyncXMLFormat *xmlformat1, OSyncXMLF
 			} while(res == 0);			
 			
 			do {
-				/* get the points*/
-				int p = _osync_xmlformat_get_points(points, &cur_pos, basic_points, curfieldname);
-				
+			
 				/* if same then compare and give points*/
 				do {
 					if(!same)
@@ -523,6 +581,7 @@ OSyncConvCmpResult osync_xmlformat_compare(OSyncXMLFormat *xmlformat1, OSyncXMLF
 						
 						if(same) {
 							/* add the points */
+							osync_trace(TRACE_INTERNAL, "add %i point(s) for same fields: %s", p, curfieldname);
 							collected_points += p;
 							fieldlist1 = g_slist_delete_link(fieldlist1, cur_list1);
 							fieldlist2 = g_slist_delete_link(fieldlist2, cur_list2);
@@ -570,11 +629,13 @@ OSyncConvCmpResult osync_xmlformat_compare(OSyncXMLFormat *xmlformat1, OSyncXMLF
 						/* add or subtract the points */
 						if(found) {
 							/* add the points */
+							osync_trace(TRACE_INTERNAL, "add %i point(s) for similiar field: %s", p, curfieldname);
 							collected_points += p;
 							fieldlist1 = g_slist_delete_link(fieldlist1, cur_list1);
 							fieldlist2 = g_slist_delete_link(fieldlist2, cur_list2);
 						}else{
 							/* subtract the points */
+							osync_trace(TRACE_INTERNAL, "subtracting %i point(s) for missing field: %s", p, curfieldname);
 							collected_points -= p;
 							subtracted_count++;
 							fieldlist1 = g_slist_delete_link(fieldlist1, cur_list1);
@@ -584,10 +645,12 @@ OSyncConvCmpResult osync_xmlformat_compare(OSyncXMLFormat *xmlformat1, OSyncXMLF
 					/* subtract the points for the remaining elements in the list2 */
 					while(fieldlist2) {
 						/* subtract the points */
-						if(subtracted_count > 0)
+						if(subtracted_count > 0) {
 							subtracted_count--;
-						else
+						} else {
+							osync_trace(TRACE_INTERNAL, "subtracting %i point(s) for remaining field: %s", p, curfieldname);
 							collected_points -= p;
+						}
 						fieldlist2 = g_slist_delete_link(fieldlist2, fieldlist2);
 					}
 	
@@ -601,12 +664,12 @@ OSyncConvCmpResult osync_xmlformat_compare(OSyncXMLFormat *xmlformat1, OSyncXMLF
 		}
 	};
 	
-	osync_trace(TRACE_INTERNAL, "Result is: %i, Treshold is: %i", collected_points, treshold);
+	osync_trace(TRACE_INTERNAL, "Result is: %i, Treshold is: %i", collected_points, threshold);
 	if (same) {
 		osync_trace(TRACE_EXIT, "%s: SAME", __func__);
 		return OSYNC_CONV_DATA_SAME;
 	}
-	if (collected_points >= treshold) {
+	if (collected_points >= threshold) {
 		osync_trace(TRACE_EXIT, "%s: SIMILAR", __func__);
 		return OSYNC_CONV_DATA_SIMILAR;
 	}
@@ -614,4 +677,16 @@ OSyncConvCmpResult osync_xmlformat_compare(OSyncXMLFormat *xmlformat1, OSyncXMLF
 	return OSYNC_CONV_DATA_MISMATCH;
 }
 
+/**
+ * @brief Returns the size of the OSyncXMLFormat struct.
+ *
+ * This is needed since the struct itself is private.
+ *
+ * @return The size of OSyncXMLFormat struct. 
+ */
+unsigned int osync_xmlformat_size()
+{
+	return sizeof(OSyncXMLFormat);
+}
+
 /*@}*/
