@@ -35,7 +35,7 @@
 
 /*! @brief Registers a new plugin
  *
- * This function creates a new OSyncPluginInfo object, that
+ * This function creates a new OSyncPlugin object, that
  * can be used to register a new plugin dynamically. This
  * can be used by a module to register multiple plugins,
  * instead of using the get_info() function which allows
@@ -53,6 +53,11 @@ OSyncPlugin *osync_plugin_new(OSyncError **error)
 	plugin->config_type = OSYNC_PLUGIN_NEEDS_CONFIGURATION;
 	plugin->start_type = OSYNC_START_TYPE_THREAD;
 	plugin->ref_count = 1;
+
+	plugin->timeout.initialize = OSYNC_PLUGIN_TIMEOUT_INITIALIZE;
+	plugin->timeout.finalize = OSYNC_PLUGIN_TIMEOUT_FINALIZE;
+	plugin->timeout.discover = OSYNC_PLUGIN_TIMEOUT_DISCOVER;
+	plugin->timeout.useable = OSYNC_PLUGIN_TIMEOUT_USEABLE;
 	
 	return plugin;
 }
@@ -276,13 +281,6 @@ void osync_plugin_set_discover(OSyncPlugin *plugin, discover_fn discover)
 	plugin->discover = discover;
 }
 
-void osync_plugin_set_objtypes(OSyncPlugin *plugin, finalize_fn fin)
-{
-	osync_assert(plugin);
-	
-	plugin->finalize = fin;
-}
-
 void *osync_plugin_initialize(OSyncPlugin *plugin, OSyncPluginInfo *info, OSyncError **error)
 {
 	osync_assert(plugin);
@@ -324,4 +322,46 @@ osync_bool osync_plugin_is_usable(OSyncPlugin *plugin, OSyncError **error)
 	return TRUE;
 }
 
+void osync_plugin_set_discover_timeout(OSyncPlugin *plugin, unsigned int timeout)
+{
+	osync_assert(plugin);
+	plugin->timeout.discover = timeout;
+}
+
+unsigned int osync_plugin_get_initialize_timeout(OSyncPlugin *plugin)
+{
+	osync_assert(plugin);
+	return plugin->timeout.initialize;
+}
+
+void osync_plugin_set_initialize_timeout(OSyncPlugin *plugin, unsigned int timeout)
+{
+	osync_assert(plugin);
+	plugin->timeout.initialize = timeout;
+}
+
+unsigned int osync_plugin_get_finalize_timeout(OSyncPlugin *plugin)
+{
+	osync_assert(plugin);
+	return plugin->timeout.finalize;
+}
+
+void osync_plugin_set_finalize_timeout(OSyncPlugin *plugin, unsigned int timeout)
+{
+	osync_assert(plugin);
+	plugin->timeout.finalize = timeout;
+}
+
+unsigned int osync_plugin_get_useable_timeout(OSyncPlugin *plugin)
+{
+	osync_assert(plugin);
+	return plugin->timeout.useable;
+}
+
+void osync_plugin_set_useable_timeout(OSyncPlugin *plugin, unsigned int timeout)
+{
+	osync_assert(plugin);
+	plugin->timeout.useable = timeout;
+}
+
 /*@}*/
