@@ -6,7 +6,10 @@ struct OSyncPluginInfo {
 	char *config;
 	GList *objtypes;
 	char *configdir;
-	OSyncObjTypeSink *sink;
+
+	/* The main sink */
+	OSyncObjTypeSink *main_sink;
+
 	OSyncObjTypeSink *current_sink;
 	OSyncFormatEnv *formatenv;
 	//devinfo
@@ -14,6 +17,10 @@ struct OSyncPluginInfo {
 	char *groupname;
 	OSyncVersion *version;
 	OSyncCapabilities *capabilities;
+
+#ifdef OPENSYNC_UNITTESTS
+	long long int memberid; // introduced only for testing purpose (mock-sync)
+#endif	
 };
 
 #endif /*OPENSYNC_PLUGIN_INFO_INTERNALS_H_*/
