@@ -33,6 +33,23 @@
  */
 /*@{*/
 
+/*! @brief Creates a new main sink
+ *
+ * Main sink is objtype neutrual and should be used for object type
+ * neutral actions. Actions like connecting/disconnecting which could
+ * be a object type neutral - e.g. connecting to a device via bluetooth.
+ * Object type specific example would be a connection to different 
+ * object type specific databases/ressources.
+ *
+ * Main sink is not limited to the connect and disconnect functions! 
+ * 
+ * @param error Pointer to an error struct
+ * @returns the newly created main sink
+ */
+OSyncObjTypeSink *osync_objtype_main_sink_new(OSyncError **error)
+{
+	return osync_objtype_sink_new(NULL, error);
+}
 
 /*! @brief Creates a new sink for an object type
  *
@@ -54,7 +71,9 @@ OSyncObjTypeSink *osync_objtype_sink_new(const char *objtype, OSyncError **error
 	sink->write = TRUE;
 
 	sink->enabled = TRUE;
-	
+
+	memset(&sink->timeout, 0, sizeof(sink->timeout));
+
 	return sink;
 }
 
@@ -79,13 +98,19 @@ OSyncObjTypeSink *osync_objtype_sink_ref(OSyncObjTypeSink *sink)
  */
 void osync_objtype_sink_unref(OSyncObjTypeSink *sink)
 {
+	OSyncList *f = NULL;
 	osync_assert(sink);
 	
 	if (g_atomic_int_dec_and_test(&(sink->ref_count))) {
 		while (sink->objformats) {
-			char *format = sink->objformats->data;
+			char **format_vertice = sink->objformats->data;
+			char *format = format_vertice[0];
+			char *format_config = format_vertice[1];
 			g_free(format);
-			sink->objformats = osync_list_remove(sink->objformats, format);
+			if(format_config)
+			      g_free(format_config);
+			g_free(format_vertice);
+			sink->objformats = osync_list_remove(sink->objformats, sink->objformats->data);
 		}
 		
 		if (sink->objtype)
@@ -126,7 +151,8 @@ static osync_bool _osync_objtype_sink_find_objformat(OSyncObjTypeSink *sink, con
 	osync_assert(sink);
 	OSyncList *f = sink->objformats;
 	for (; f; f = f->next) {
-		if (!strcmp(f->data, format))
+		const char **format_vertice = f->data;
+		if (!strcmp(format_vertice[0], format))
 			return TRUE;
 	}
 	return FALSE;
@@ -154,7 +180,28 @@ int osync_objtype_sink_num_objformats(OSyncObjTypeSink *sink)
 const char *osync_objtype_sink_nth_objformat(OSyncObjTypeSink *sink, int nth)
 {
 	osync_assert(sink);
-	return osync_list_nth_data(sink->objformats, nth);
+	char **format_vertice = osync_list_nth_data(sink->objformats, nth);
+	if (format_vertice)
+		return format_vertice[0];
+	else
+		return NULL;
+}
+
+/*! @brief Returns the nth object format conversion config in the sink
+ * 
+ * @param sink Pointer to the sink
+ * @param nth the index of the object format conversion config to return
+ * @returns the conversion config of the object format at the specified index
+ * 
+ */
+const char *osync_objtype_sink_nth_objformat_config(OSyncObjTypeSink *sink, int nth)
+{
+	osync_assert(sink);
+	char **format_vertice = osync_list_nth_data(sink->objformats, nth);
+	if (format_vertice) 
+		return format_vertice[1];
+	else
+		return NULL;
 }
 
 /** @brief Returns the list of object formats in the sink
@@ -180,8 +227,33 @@ void osync_objtype_sink_add_objformat(OSyncObjTypeSink *sink, const char *format
 	osync_assert(sink);
 	osync_assert(format);
 	
-	if (!_osync_objtype_sink_find_objformat(sink, format))
-		sink->objformats = osync_list_append(sink->objformats, g_strdup(format));
+	if (!_osync_objtype_sink_find_objformat(sink, format)) {
+	        char **format_vertice = g_malloc0(2*sizeof(char **));
+		format_vertice[0] = g_strdup(format);
+		format_vertice[1] = NULL;
+		sink->objformats = osync_list_append(sink->objformats, format_vertice);
+	}
+}
+
+/** @brief Adds an object format with its conversion config to the sink
+ * 
+ * @param sink Pointer to the sink
+ * @param format name of the object format
+ * @param format_config config of the object format for the conversion path
+ * 
+ */
+void osync_objtype_sink_add_objformat_with_config(OSyncObjTypeSink *sink, const char *format, const char *format_config)
+{
+	osync_assert(sink);
+	osync_assert(format);
+	
+	if (!_osync_objtype_sink_find_objformat(sink, format)) {
+	        char **format_vertice = g_malloc0(2*sizeof(char **));
+		format_vertice[0] = g_strdup(format);
+		format_vertice[1] = g_strdup(format_config);
+		osync_trace(TRACE_INTERNAL, "CONFIG %s", format_vertice[1]);
+		sink->objformats = osync_list_append(sink->objformats, format_vertice);
+	}
 }
 
 /** @brief Removes an object format from the sink
@@ -196,7 +268,8 @@ void osync_objtype_sink_remove_objformat(OSyncObjTypeSink *sink, const char *for
 	osync_assert(sink);
 	osync_assert(format);
 	for (f = sink->objformats; f; f = f->next) {
-		if (!strcmp((char *)f->data, format)) {
+		const char **format_vertice = f->data;
+		if (!strcmp(format_vertice[0], format)) {
 			sink->objformats = osync_list_remove(sink->objformats, f->data);
 			break;
 		}
@@ -329,12 +402,16 @@ void osync_objtype_sink_get_changes(OSyncObjTypeSink *sink, void *plugindata, OS
 	osync_assert(ctx);
 	
 	functions = sink->functions;
-	if (!functions.get_changes) {
+	if (sink->objtype && !functions.get_changes) {
 		osync_context_report_error(ctx, OSYNC_ERROR_GENERIC, "No get_changeinfo function was given");
 		osync_trace(TRACE_EXIT_ERROR, "%s: No get_changes function was given", __func__);
 		return;
+	} else if (!functions.get_changes) {
+		osync_context_report_success(ctx);
+	} else {
+		functions.get_changes(plugindata, info, ctx);
 	}
-	functions.get_changes(plugindata, info, ctx);
+
 	osync_trace(TRACE_EXIT, "%s", __func__);
 }
 
@@ -358,12 +435,18 @@ void osync_objtype_sink_read_change(OSyncObjTypeSink *sink, void *plugindata, OS
 	osync_assert(change);
 	
 	functions = sink->functions;
-	if (!functions.read) {
+
+
+	if (sink->objtype && !functions.read) {
 		osync_context_report_error(ctx, OSYNC_ERROR_GENERIC, "No read function was given");
 		osync_trace(TRACE_EXIT_ERROR, "%s: No read function was given", __func__);
 		return;
+	} else if (!functions.read) {
+		osync_context_report_success(ctx);
+	} else {
+		functions.read(plugindata, info, ctx, change);
 	}
-	functions.read(plugindata, info, ctx, change);
+
 	osync_trace(TRACE_EXIT, "%s", __func__);
 }
 
@@ -386,11 +469,11 @@ void osync_objtype_sink_connect(OSyncObjTypeSink *sink, void *plugindata, OSyncP
 	
 	functions = sink->functions;
 	if (!functions.connect) {
-		osync_context_report_error(ctx, OSYNC_ERROR_GENERIC, "No connect function was given");
-		osync_trace(TRACE_EXIT_ERROR, "%s: No connect function was given", __func__);
-		return;
+		osync_context_report_success(ctx);
+	} else {
+		functions.connect(plugindata, info, ctx);
 	}
-	functions.connect(plugindata, info, ctx);
+
 	osync_trace(TRACE_EXIT, "%s", __func__);
 }
 
@@ -413,11 +496,11 @@ void osync_objtype_sink_disconnect(OSyncObjTypeSink *sink, void *plugindata, OSy
 	
 	functions = sink->functions;
 	if (!functions.disconnect) {
-		osync_context_report_error(ctx, OSYNC_ERROR_GENERIC, "No disconnect function was given");
-		osync_trace(TRACE_EXIT_ERROR, "%s: No disconnect function was given", __func__);
-		return;
+		osync_context_report_success(ctx);
+	} else {
+		functions.disconnect(plugindata, info, ctx);
 	}
-	functions.disconnect(plugindata, info, ctx);
+
 	osync_trace(TRACE_EXIT, "%s", __func__);
 }
 
@@ -481,12 +564,15 @@ void osync_objtype_sink_commit_change(OSyncObjTypeSink *sink, void *plugindata,
 		return;
 	} else {
 		// Send the change
-		if (!functions.commit) {
+		if (sink->objtype && !functions.commit) {
 			osync_context_report_error(ctx, OSYNC_ERROR_GENERIC, "No commit_change function was given");
 			osync_trace(TRACE_EXIT_ERROR, "%s: No commit_change function was given", __func__);
 			return;
+		} else if (!functions.commit) {
+			osync_context_report_success(ctx);
+		} else {
+			functions.commit(plugindata, info, ctx, change);
 		}
-		functions.commit(plugindata, info, ctx, change);
 	}
 
 	osync_trace(TRACE_EXIT, "%s", __func__);
@@ -726,4 +812,166 @@ void osync_objtype_sink_set_slowsync(OSyncObjTypeSink *sink, osync_bool slowsync
 	sink->slowsync = slowsync;
 }
 
+void osync_objtype_sink_set_connect_timeout(OSyncObjTypeSink *sink, unsigned int timeout)
+{
+	osync_assert(sink);
+	sink->timeout.connect = timeout;
+}
+
+unsigned int osync_objtype_sink_get_connect_timeout_or_default(OSyncObjTypeSink *sink)
+{
+	osync_assert(sink);
+	return sink->timeout.connect ? sink->timeout.connect : OSYNC_SINK_TIMEOUT_CONNECT;
+}
+
+unsigned int osync_objtype_sink_get_connect_timeout(OSyncObjTypeSink *sink)
+{
+	osync_assert(sink);
+	return sink->timeout.connect;
+}
+
+void osync_objtype_sink_set_disconnect_timeout(OSyncObjTypeSink *sink, unsigned int timeout)
+{
+	osync_assert(sink);
+	sink->timeout.disconnect = timeout;
+}
+
+unsigned int osync_objtype_sink_get_disconnect_timeout_or_default(OSyncObjTypeSink *sink)
+{
+	osync_assert(sink);
+	return sink->timeout.disconnect ? sink->timeout.disconnect : OSYNC_SINK_TIMEOUT_DISCONNECT;
+}
+
+unsigned int osync_objtype_sink_get_disconnect_timeout(OSyncObjTypeSink *sink)
+{
+	osync_assert(sink);
+	return sink->timeout.disconnect;
+}
+
+void osync_objtype_sink_set_getchanges_timeout(OSyncObjTypeSink *sink, unsigned int timeout)
+{
+	osync_assert(sink);
+	sink->timeout.get_changes = timeout;
+}
+
+unsigned int osync_objtype_sink_get_getchanges_timeout_or_default(OSyncObjTypeSink *sink)
+{
+	osync_assert(sink);
+	return sink->timeout.commit ? sink->timeout.get_changes : OSYNC_SINK_TIMEOUT_GETCHANGES;
+}
+
+unsigned int osync_objtype_sink_get_getchanges_timeout(OSyncObjTypeSink *sink)
+{
+	osync_assert(sink);
+	return sink->timeout.get_changes;
+}
+
+void osync_objtype_sink_set_commit_timeout(OSyncObjTypeSink *sink, unsigned int timeout)
+{
+	osync_assert(sink);
+	sink->timeout.commit = timeout;
+}
+
+unsigned int osync_objtype_sink_get_commit_timeout_or_default(OSyncObjTypeSink *sink)
+{
+	osync_assert(sink);
+	return sink->timeout.commit ? sink->timeout.commit : OSYNC_SINK_TIMEOUT_COMMIT;
+}
+
+unsigned int osync_objtype_sink_get_commit_timeout(OSyncObjTypeSink *sink)
+{
+	osync_assert(sink);
+	return sink->timeout.commit;
+}
+
+void osync_objtype_sink_set_batchcommit_timeout(OSyncObjTypeSink *sink, unsigned int timeout)
+{
+	osync_assert(sink);
+	sink->timeout.batch_commit = timeout;
+}
+
+unsigned int osync_objtype_sink_get_batchcommit_timeout_or_default(OSyncObjTypeSink *sink)
+{
+	osync_assert(sink);
+	return sink->timeout.batch_commit ? sink->timeout.batch_commit : OSYNC_SINK_TIMEOUT_BATCHCOMMIT;
+}
+
+unsigned int osync_objtype_sink_get_batchcommit_timeout(OSyncObjTypeSink *sink)
+{
+	osync_assert(sink);
+	return sink->timeout.batch_commit;
+}
+
+void osync_objtype_sink_set_committedall_timeout(OSyncObjTypeSink *sink, unsigned int timeout)
+{
+	osync_assert(sink);
+	sink->timeout.committed_all = timeout;
+}
+
+unsigned int osync_objtype_sink_get_committedall_timeout_or_default(OSyncObjTypeSink *sink)
+{
+	osync_assert(sink);
+	return sink->timeout.committed_all ? sink->timeout.committed_all : OSYNC_SINK_TIMEOUT_COMMITTEDALL;
+}
+
+unsigned int osync_objtype_sink_get_committedall_timeout(OSyncObjTypeSink *sink)
+{
+	osync_assert(sink);
+	return sink->timeout.committed_all;
+}
+
+void osync_objtype_sink_set_syncdone_timeout(OSyncObjTypeSink *sink, unsigned int timeout)
+{
+	osync_assert(sink);
+	sink->timeout.sync_done = timeout;
+}
+
+unsigned int osync_objtype_sink_get_syncdone_timeout_or_default(OSyncObjTypeSink *sink)
+{
+	osync_assert(sink);
+	return sink->timeout.sync_done ? sink->timeout.sync_done : OSYNC_SINK_TIMEOUT_SYNCDONE;
+}
+
+unsigned int osync_objtype_sink_get_syncdone_timeout(OSyncObjTypeSink *sink)
+{
+	osync_assert(sink);
+	return sink->timeout.sync_done;
+}
+
+void osync_objtype_sink_set_write_timeout(OSyncObjTypeSink *sink, unsigned int timeout)
+{
+	osync_assert(sink);
+	sink->timeout.write = timeout;
+}
+
+unsigned int osync_objtype_sink_get_write_timeout_or_default(OSyncObjTypeSink *sink)
+{
+	osync_assert(sink);
+	return sink->timeout.write ? sink->timeout.write : OSYNC_SINK_TIMEOUT_WRITE;
+}
+
+unsigned int osync_objtype_sink_get_write_timeout(OSyncObjTypeSink *sink)
+{
+	osync_assert(sink);
+	return sink->timeout.write;
+}
+
+void osync_objtype_sink_set_read_timeout(OSyncObjTypeSink *sink, unsigned int timeout)
+{
+	osync_assert(sink);
+	sink->timeout.read = timeout;
+}
+
+unsigned int osync_objtype_sink_get_read_timeout_or_default(OSyncObjTypeSink *sink)
+{
+	osync_assert(sink);
+	return sink->timeout.read ? sink->timeout.read : OSYNC_SINK_TIMEOUT_READ;
+}
+
+unsigned int osync_objtype_sink_get_read_timeout(OSyncObjTypeSink *sink)
+{
+	osync_assert(sink);
+	return sink->timeout.read;
+}
+
 /*@}*/
