@@ -20,8 +20,28 @@
 
 #include "mock_sync.h"
 #include "mock_format.h"
-#include <opensync/opensync-version.h>
-#include <stdlib.h>
+
+static osync_bool mock_get_error(long long int memberid, const char *domain)
+{
+        const char *env = g_getenv(domain);
+
+        if (!env)
+                return FALSE;
+
+        int num = atoi(env);
+        int mask = 1 << (memberid - 1);
+        if (num & mask) {
+                char *chancestr = g_strdup_printf("%s_PROB", domain);
+                const char *chance = g_getenv(chancestr);
+                g_free(chancestr);
+                if (!chance)
+                        return TRUE;
+                int prob = atoi(chance);
+                if (prob >= g_random_int_range(0, 100))
+                        return TRUE;
+        }
+        return FALSE;
+}
 
 static void free_dir(OSyncFileDir *dir)
 {
@@ -49,14 +69,16 @@ static void free_env(mock_env *env)
 	g_free(env);
 }
 
-static osync_bool osync_filesync_parse_directory(mock_env *env, xmlNode *cur, OSyncError **error)
+static osync_bool mock_parse_directory(mock_env *env, OSyncPluginInfo *info, xmlNode *cur, OSyncError **error)
 {
-	osync_trace(TRACE_ENTRY, "%s(%p, %p, %p)", __func__, env, cur, error);
+	osync_trace(TRACE_ENTRY, "%s(%p, %p, %p, %p)", __func__, env, info, cur, error);
 
 	OSyncFileDir *dir = osync_try_malloc0(sizeof(OSyncFileDir), error);
 	if (!dir)
 		goto error;
 	dir->env = env;
+
+	OSyncFormatEnv *formatenv = osync_plugin_info_get_format_env(info);
 	
 	while (cur != NULL) {
 		char *str = (char*)xmlNodeGetContent(cur);
@@ -65,6 +87,8 @@ static osync_bool osync_filesync_parse_directory(mock_env *env, xmlNode *cur, OS
 				dir->path = g_strdup(str);
 			} else if (!xmlStrcmp(cur->name, (const xmlChar *)"objtype")) {
 				dir->objtype = g_strdup(str);
+			} else if (!xmlStrcmp(cur->name, (const xmlChar *)"objformat")) {
+				dir->objformat = osync_format_env_find_objformat(formatenv, str);
 			} else if (!xmlStrcmp(cur->name, (const xmlChar *)"recursive")) {
 				dir->recursive = (g_ascii_strcasecmp(str, "TRUE") == 0);
 			}
@@ -93,9 +117,10 @@ error:
 }
 
 /*Load the state from a xml file and return it in the conn struct*/
-static osync_bool osync_filesync_parse_settings(mock_env *env, const char *data, OSyncError **error)
+static osync_bool mock_parse_settings(mock_env *env, OSyncPluginInfo *info, OSyncError **error)
 {
-	osync_trace(TRACE_ENTRY, "%s(%p, %p, %p)", __func__, env, data, error);
+	osync_trace(TRACE_ENTRY, "%s(%p, %p, %p)", __func__, env, info, error);
+	const char *data = osync_plugin_info_get_config(info);
 	xmlDoc *doc = NULL;
 	xmlNode *cur = NULL;
 
@@ -123,7 +148,7 @@ static osync_bool osync_filesync_parse_settings(mock_env *env, const char *data,
 		char *str = (char*)xmlNodeGetContent(cur);
 		if (str) {
 			if (!xmlStrcmp(cur->name, (const xmlChar *)"directory")) {
-				if (!osync_filesync_parse_directory(env, cur->xmlChildrenNode, error))
+				if (!mock_parse_directory(env, info, cur->xmlChildrenNode, error))
 					goto error_free_doc;
 			}
 			xmlFree(str);
@@ -143,20 +168,34 @@ error:
 	return FALSE;
 }
 
-static char *osync_filesync_generate_hash(struct stat *buf)
+static char *mock_generate_hash(struct stat *buf)
 {
 	char *hash = g_strdup_printf("%i-%i", (int)buf->st_mtime, (int)buf->st_ctime);
 	return hash;
 }
 
-static void osync_filesync_connect(void *data, OSyncPluginInfo *info, OSyncContext *ctx)
+static void mock_connect(void *data, OSyncPluginInfo *info, OSyncContext *ctx)
 {
 	OSyncError *error = NULL;
-	
-	osync_trace(TRACE_ENTRY, "%s(%p, %p, %p)", __func__, data, info, ctx);
 	OSyncObjTypeSink *sink = osync_plugin_info_get_sink(info);
 	OSyncFileDir *dir = osync_objtype_sink_get_userdata(sink);
+
+        dir->committed_all = TRUE;
 	
+	osync_trace(TRACE_ENTRY, "%s(%p, %p, %p)", __func__, data, info, ctx);
+	
+	if (mock_get_error(info->memberid, "CONNECT_ERROR")) {
+		osync_context_report_error(ctx, OSYNC_ERROR_EXPECTED, "Triggering CONNECT_ERROR error");
+		osync_trace(TRACE_EXIT_ERROR, "%s: %s", __func__, "Triggering CONNECT_ERROR error");
+		return;
+	}
+
+	if (mock_get_error(info->memberid, "CONNECT_TIMEOUT")) {
+		/* Don't report context back ... let it timeout! */
+		osync_trace(TRACE_EXIT_ERROR, "%s: %s", __func__, "Triggering CONNECT_TIMEOUT error");
+		return;
+	}
+
 	osync_trace(TRACE_INTERNAL, "The configdir: %s", osync_plugin_info_get_configdir(info));
 	char *tablepath = g_strdup_printf("%s/hashtable.db", osync_plugin_info_get_configdir(info));
 	dir->hashtable = osync_hashtable_new(tablepath, osync_objtype_sink_get_name(sink), &error);
@@ -188,11 +227,23 @@ error:
 	osync_error_unref(&error);
 }
 
-static void osync_filesync_disconnect(void *data, OSyncPluginInfo *info, OSyncContext *ctx)
+static void mock_disconnect(void *data, OSyncPluginInfo *info, OSyncContext *ctx)
 {
 	osync_trace(TRACE_ENTRY, "%s(%p, %p, %p)", __func__, data, info, ctx);
 	OSyncObjTypeSink *sink = osync_plugin_info_get_sink(info);
 	OSyncFileDir *dir = osync_objtype_sink_get_userdata(sink);
+
+	if (!g_getenv("NO_COMMITTED_ALL_CHECK"))
+		fail_unless(dir->committed_all == TRUE, NULL);
+	dir->committed_all = FALSE;
+
+	if (mock_get_error(info->memberid, "DISCONNECT_ERROR")) {
+		osync_context_report_error(ctx, OSYNC_ERROR_EXPECTED, "Triggering DISCONNECT_ERROR error");
+		return;
+	}
+	if (mock_get_error(info->memberid, "DISCONNECT_TIMEOUT"))
+		return;
+
 	
 	if (dir->hashtable) {
 		osync_hashtable_free(dir->hashtable);
@@ -209,12 +260,11 @@ static void osync_filesync_disconnect(void *data, OSyncPluginInfo *info, OSyncCo
 //typedef void (* OSyncSinkCommittedAllFn) (void *data, OSyncPluginInfo *info, OSyncContext *ctx);
 
 
-static osync_bool osync_filesync_read(void *data, OSyncPluginInfo *info, OSyncContext *ctx, OSyncChange *change)
+static osync_bool mock_read(void *data, OSyncPluginInfo *info, OSyncContext *ctx, OSyncChange *change)
 {
 	osync_trace(TRACE_ENTRY, "%s(%p, %p, %p, %p)", __func__, data, info, ctx, change);
 	OSyncObjTypeSink *sink = osync_plugin_info_get_sink(info);
 	OSyncFileDir *dir = osync_objtype_sink_get_userdata(sink);
-	mock_env *env = (mock_env *)data;
 	OSyncError *error = NULL;
 	
 	char *filename = g_strdup_printf("%s/%s", dir->path, osync_change_get_uid(change));
@@ -234,7 +284,7 @@ static osync_bool osync_filesync_read(void *data, OSyncPluginInfo *info, OSyncCo
 	if (!osync_file_read(filename, &(file->data), &(file->size), &error))
 		goto error_free_file;
 	
-	OSyncData *odata = osync_data_new((char *)file, sizeof(OSyncFileFormat), env->objformat, &error);
+	OSyncData *odata = osync_data_new((char *)file, sizeof(OSyncFileFormat), dir->objformat, &error);
 	if (!odata)
 		goto error_free_data;
 
@@ -264,7 +314,7 @@ error:
 	return FALSE;
 }
 
-static osync_bool osync_filesync_write(void *data, OSyncPluginInfo *info, OSyncContext *ctx, OSyncChange *change)
+static osync_bool mock_write(void *data, OSyncPluginInfo *info, OSyncContext *ctx, OSyncChange *change)
 {
 	osync_trace(TRACE_ENTRY, "%s(%p, %p, %p, %p)", __func__, data, info, ctx, change);
 	OSyncObjTypeSink *sink = osync_plugin_info_get_sink(info);
@@ -330,7 +380,7 @@ error:
  *            start with a slash. See note above.
  *
  */
-static void osync_filesync_report_dir(OSyncFileDir *directory, const char *subdir, OSyncContext *ctx)
+static void mock_report_dir(OSyncFileDir *directory, const char *subdir, OSyncContext *ctx, OSyncPluginInfo *info)
 {
 	GError *gerror = NULL;
 	const char *de = NULL;
@@ -369,12 +419,12 @@ static void osync_filesync_report_dir(OSyncFileDir *directory, const char *subdi
 		if (g_file_test(filename, G_FILE_TEST_IS_DIR)) {
 			/* Recurse into subdirectories */
 			if (directory->recursive)
-				osync_filesync_report_dir(directory, relative_filename, ctx);
+				mock_report_dir(directory, relative_filename, ctx, info);
 		} else if (g_file_test(filename, G_FILE_TEST_IS_REGULAR)) {
 			
 			struct stat buf;
 			stat(filename, &buf);
-			char *hash = osync_filesync_generate_hash(&buf);
+			char *hash = mock_generate_hash(&buf);
 			
 			OSyncChangeType type = osync_hashtable_get_changetype(directory->hashtable, relative_filename, hash);
 			if (type == OSYNC_CHANGE_TYPE_UNMODIFIED) {
@@ -414,20 +464,30 @@ static void osync_filesync_report_dir(OSyncFileDir *directory, const char *subdi
 			file->path = g_strdup(relative_filename);
 			
 			OSyncError *error = NULL;
-			if (!osync_file_read(filename, &(file->data), &(file->size), &error)) {
-				osync_change_unref(change);
-				osync_context_report_osyncwarning(ctx, error);
-				osync_error_unref(&error);
-				g_free(filename);
-				continue;
-			}
-			
-			OSyncData *odata = osync_data_new((char *)file, sizeof(OSyncFileFormat), directory->env->objformat, &error);
-			if (!odata) {
-				osync_change_unref(change);
-				osync_context_report_osyncwarning(ctx, error);
-				osync_error_unref(&error);
-				continue;
+			OSyncData *odata = NULL;
+
+			if (mock_get_error(info->memberid, "ONLY_INFO")) {
+				odata = osync_data_new(NULL, 0, directory->objformat, &error);
+			} else {
+				if (!osync_file_read(filename, &(file->data), &(file->size), &error)) {
+					osync_change_unref(change);
+					osync_context_report_osyncwarning(ctx, error);
+					osync_error_unref(&error);
+					g_free(filename);
+					continue;
+				}
+
+				if (mock_get_error(info->memberid, "SLOW_REPORT"))
+					sleep(1);
+				
+				odata = osync_data_new((char *)file, sizeof(OSyncFileFormat), directory->objformat, &error);
+				if (!odata) {
+					osync_change_unref(change);
+					osync_context_report_osyncwarning(ctx, error);
+					osync_error_unref(&error);
+					continue;
+				}
+
 			}
 			
 			osync_data_set_objtype(odata, osync_objtype_sink_get_name(directory->sink));
@@ -450,15 +510,31 @@ static void osync_filesync_report_dir(OSyncFileDir *directory, const char *subdi
 	osync_trace(TRACE_EXIT, "%s", __func__);
 }
 
-static void osync_filesync_get_changes(void *data, OSyncPluginInfo *info, OSyncContext *ctx)
+static void mock_get_changes(void *data, OSyncPluginInfo *info, OSyncContext *ctx)
 {
 	osync_trace(TRACE_ENTRY, "%s(%p, %p, %p)", __func__, data, info, ctx);
 	OSyncObjTypeSink *sink = osync_plugin_info_get_sink(info);
 	OSyncFileDir *dir = osync_objtype_sink_get_userdata(sink);
-	mock_env *env = (mock_env *)data;
 	int i = 0;
 	OSyncError *error = NULL;
-	
+
+	fail_unless(dir->committed_all == TRUE, NULL);
+	dir->committed_all = FALSE;
+
+	if (mock_get_error(info->memberid, "GET_CHANGES_ERROR")) {
+		osync_context_report_error(ctx, OSYNC_ERROR_EXPECTED, "Triggering GET_CHANGES_ERROR error");
+		osync_trace(TRACE_EXIT_ERROR, "%s - Triggering GET_CHANGES error", __func__);
+		return;
+	}
+
+	if (mock_get_error(info->memberid, "GET_CHANGES_TIMEOUT")) {
+		osync_trace(TRACE_EXIT, "%s - Triggering GET_CHANGES_TIMEOUT (without context report!)", __func__);
+		return;
+	}
+
+	if (mock_get_error(info->memberid, "GET_CHANGES_TIMEOUT2"))
+		sleep(8);
+		
 	if (osync_objtype_sink_get_slowsync(dir->sink)) {
 		osync_trace(TRACE_INTERNAL, "Slow sync requested");
 		osync_hashtable_reset(dir->hashtable);
@@ -466,7 +542,7 @@ static void osync_filesync_get_changes(void *data, OSyncPluginInfo *info, OSyncC
 	
 	osync_trace(TRACE_INTERNAL, "get_changes for %s", osync_objtype_sink_get_name(sink));
 
-	osync_filesync_report_dir(dir, NULL, ctx);
+	mock_report_dir(dir, NULL, ctx, info);
 	
 	char **uids = osync_hashtable_get_deleted(dir->hashtable);
 	for (i = 0; uids[i]; i++) {
@@ -481,7 +557,7 @@ static void osync_filesync_get_changes(void *data, OSyncPluginInfo *info, OSyncC
 		osync_change_set_uid(change, uids[i]);
 		osync_change_set_changetype(change, OSYNC_CHANGE_TYPE_DELETED);
 		
-		OSyncData *odata = osync_data_new(NULL, 0, env->objformat, &error);
+		OSyncData *odata = osync_data_new(NULL, 0, dir->objformat, &error);
 		if (!odata) {
 			g_free(uids[i]);
 			osync_change_unref(change);
@@ -507,15 +583,26 @@ static void osync_filesync_get_changes(void *data, OSyncPluginInfo *info, OSyncC
 	osync_trace(TRACE_EXIT, "%s", __func__);
 }
 
-static void osync_filesync_commit_change(void *data, OSyncPluginInfo *info, OSyncContext *ctx, OSyncChange *change)
+static void mock_commit_change(void *data, OSyncPluginInfo *info, OSyncContext *ctx, OSyncChange *change)
 {
 	osync_trace(TRACE_ENTRY, "%s", __func__);
 	OSyncObjTypeSink *sink = osync_plugin_info_get_sink(info);
 	OSyncFileDir *dir = osync_objtype_sink_get_userdata(sink);
 	
 	char *filename = NULL;
+
+	fail_unless(dir->committed_all == FALSE, NULL);
+
+	if (mock_get_error(info->memberid, "COMMIT_ERROR")) {
+		osync_context_report_error(ctx, OSYNC_ERROR_EXPECTED, "Triggering COMMIT_ERROR error");
+		return;
+	}
+	if (mock_get_error(info->memberid, "COMMIT_TIMEOUT")) {
+		osync_trace(TRACE_EXIT_ERROR, "COMMIT_TIMEOUT (mock-sync)!");
+		return;
+	}
 	
-	if (!osync_filesync_write(data, info, ctx, change)) {
+	if (!mock_write(data, info, ctx, change)) {
 		osync_trace(TRACE_EXIT_ERROR, "%s", __func__);
 		return;
 	}
@@ -526,7 +613,7 @@ static void osync_filesync_commit_change(void *data, OSyncPluginInfo *info, OSyn
 	if (osync_change_get_changetype(change) != OSYNC_CHANGE_TYPE_DELETED) {
 		struct stat buf;
 		stat(filename, &buf);
-		hash = osync_filesync_generate_hash(&buf);
+		hash = mock_generate_hash(&buf);
 	}
 	g_free(filename);
 
@@ -538,12 +625,81 @@ static void osync_filesync_commit_change(void *data, OSyncPluginInfo *info, OSyn
 	osync_trace(TRACE_EXIT, "%s", __func__);
 }
 
-static void osync_filesync_sync_done(void *data, OSyncPluginInfo *info, OSyncContext *ctx)
+static void mock_batch_commit(void *data, OSyncPluginInfo *info, OSyncContext *context, OSyncContext **contexts, OSyncChange **changes)
+{
+        osync_trace(TRACE_ENTRY, "%s(%p, %p, %p)", __func__, context, contexts, changes);
+	OSyncObjTypeSink *sink = osync_plugin_info_get_sink(info);
+	OSyncFileDir *dir = osync_objtype_sink_get_userdata(sink);
+
+        fail_unless(dir->committed_all == FALSE, NULL);
+        dir->committed_all = TRUE;
+
+        int i; 
+        for (i = 0; contexts[i]; i++) {
+                if (mock_write(data, info, contexts[i], changes[i])) {
+			char *filename = g_strdup_printf ("%s/%s", dir->path, osync_change_get_uid(changes[i]));
+			char *hash = NULL;
+	
+			if (osync_change_get_changetype(changes[i]) != OSYNC_CHANGE_TYPE_DELETED) {
+				struct stat buf;
+				stat(filename, &buf);
+				hash = mock_generate_hash(&buf);
+			}
+			g_free(filename);
+
+			osync_hashtable_update_hash(dir->hashtable, osync_change_get_changetype(changes[i]), osync_change_get_uid(changes[i]), hash);
+                        osync_context_report_success(contexts[i]);
+                }
+        }
+
+        if (g_getenv("NUM_BATCH_COMMITS")) {
+                int req = atoi(g_getenv("NUM_BATCH_COMMITS"));
+                fail_unless(req == i, NULL);
+        }
+                
+        if (mock_get_error(info->memberid, "COMMITTED_ALL_ERROR")) {
+                osync_context_report_error(context, OSYNC_ERROR_EXPECTED, "Triggering COMMITTED_ALL_ERROR error");
+                return;
+        }
+
+        osync_context_report_success(context);
+
+        osync_trace(TRACE_EXIT, "%s", __func__);
+}
+
+static void mock_committed_all(void *data, OSyncPluginInfo *info, OSyncContext *context)
+{
+        osync_trace(TRACE_ENTRY, "%s(%p)", __func__, context);
+	OSyncObjTypeSink *sink = osync_plugin_info_get_sink(info);
+	OSyncFileDir *dir = osync_objtype_sink_get_userdata(sink);
+
+        fail_unless(dir->committed_all == FALSE, NULL);
+        dir->committed_all = TRUE;
+
+        if (mock_get_error(info->memberid, "COMMITTED_ALL_ERROR")) {
+                osync_context_report_error(context, OSYNC_ERROR_EXPECTED, "Triggering COMMITTED_ALL_ERROR error");
+                osync_trace(TRACE_EXIT_ERROR, "%s: Reporting error", __func__);
+                return;
+        }
+
+        osync_context_report_success(context);
+
+        osync_trace(TRACE_EXIT, "%s", __func__);
+}
+
+static void mock_sync_done(void *data, OSyncPluginInfo *info, OSyncContext *ctx)
 {
 	osync_trace(TRACE_ENTRY, "%s(%p, %p, %p)", __func__, data, info, ctx);
 	OSyncObjTypeSink *sink = osync_plugin_info_get_sink(info);
 	OSyncFileDir *dir = osync_objtype_sink_get_userdata(sink);
 
+	if (mock_get_error(info->memberid, "SYNC_DONE_ERROR")) {
+		osync_context_report_error(ctx, OSYNC_ERROR_EXPECTED, "Triggering SYNC_DONE_ERROR error");
+		return;
+	}
+	if (mock_get_error(info->memberid, "SYNC_DONE_TIMEOUT"))
+		return;
+	
 	char *anchorpath = g_strdup_printf("%s/anchor.db", osync_plugin_info_get_configdir(info));
 	char *path_field = g_strdup_printf("path_%s", osync_objtype_sink_get_name(sink));
 	osync_anchor_update(anchorpath, path_field, dir->path);
@@ -557,10 +713,16 @@ static void osync_filesync_sync_done(void *data, OSyncPluginInfo *info, OSyncCon
 
 /* In initialize, we get the config for the plugin. Here we also must register
  * all _possible_ objtype sinks. */
-static void *osync_filesync_initialize(OSyncPlugin *plugin, OSyncPluginInfo *info, OSyncError **error)
+static void *mock_initialize(OSyncPlugin *plugin, OSyncPluginInfo *info, OSyncError **error)
 {
 	osync_trace(TRACE_ENTRY, "%s(%p, %p)", __func__, info, error);
 
+	if (mock_get_error(info->memberid, "INIT_NULL")) {
+		osync_error_set(error, OSYNC_ERROR_EXPECTED, "Triggering INIT_NULL error");
+		osync_trace(TRACE_EXIT_ERROR, "%s: %s", __func__, osync_error_print(error));
+		return NULL;
+	}
+
 	mock_env *env = osync_try_malloc0(sizeof(mock_env), error);
 	if (!env)
 		goto error;
@@ -568,9 +730,8 @@ static void *osync_filesync_initialize(OSyncPlugin *plugin, OSyncPluginInfo *inf
 	osync_trace(TRACE_INTERNAL, "The config: %s", osync_plugin_info_get_config(info));
 	
 	OSyncFormatEnv *formatenv = osync_plugin_info_get_format_env(info);
-	env->objformat = osync_format_env_find_objformat(formatenv, "mockformat1");
 	
-	if (!osync_filesync_parse_settings(env, osync_plugin_info_get_config(info), error))
+	if (!mock_parse_settings(env, info, error))
 		goto error_free_env;
 	
 	/* Now we register the objtypes that we can sync. This plugin is special. It can
@@ -586,24 +747,76 @@ static void *osync_filesync_initialize(OSyncPlugin *plugin, OSyncPluginInfo *inf
 		
 		dir->sink = sink;
 		
-		/* The file format is the only one we understand */
-		osync_objtype_sink_add_objformat(sink, "mockformat1");
+		osync_objtype_sink_add_objformat(sink, osync_objformat_get_name(dir->objformat));
 		
 		/* All sinks have the same functions of course */
 		OSyncObjTypeSinkFunctions functions;
 		memset(&functions, 0, sizeof(functions));
-		functions.connect = osync_filesync_connect;
-		functions.disconnect = osync_filesync_disconnect;
-		functions.get_changes = osync_filesync_get_changes;
-		functions.commit = osync_filesync_commit_change;
-		functions.read = osync_filesync_read;
-		functions.write = osync_filesync_write;
-		functions.sync_done = osync_filesync_sync_done;
+		functions.connect = mock_connect;
+		functions.disconnect = mock_disconnect;
+		functions.get_changes = mock_get_changes;
+
+		//Rewrite the batch commit functions so we can enable them if necessary
+		if (mock_get_error(info->memberid, "BATCH_COMMIT")) {
+			osync_trace(TRACE_INTERNAL, "Enabling batch_commit on %p:%s", sink, osync_objtype_sink_get_name(sink) ? osync_objtype_sink_get_name(sink) : "None");
+	                functions.batch_commit = mock_batch_commit; 
+		} else {
+			functions.committed_all = mock_committed_all;
+			functions.commit = mock_commit_change;
+		}
+		functions.read = mock_read;
+		functions.write = mock_write;
+		functions.sync_done = mock_sync_done;
 		
 		/* We pass the OSyncFileDir object to the sink, so we dont have to look it up
 		 * again once the functions are called */
 		osync_objtype_sink_set_functions(sink, functions, dir);
 		osync_plugin_info_add_objtype(info, sink);
+
+		//Lets reduce the timeouts a bit so the checks work faster
+		osync_objtype_sink_set_connect_timeout(sink, 5);
+		osync_objtype_sink_set_getchanges_timeout(sink, 5);
+		osync_objtype_sink_set_commit_timeout(sink, 10);
+		osync_objtype_sink_set_committedall_timeout(sink, 10);
+		osync_objtype_sink_set_batchcommit_timeout(sink, 10);
+		osync_objtype_sink_set_syncdone_timeout(sink, 5);
+		osync_objtype_sink_set_disconnect_timeout(sink, 5);
+
+		osync_objtype_sink_set_read_timeout(sink, 5);
+		osync_objtype_sink_set_write_timeout(sink, 5);
+
+
+/* XXX No testcase is currently using this at all! */
+#if 0		
+	
+		if (g_getenv("NO_TIMEOUTS")) {
+
+			/* XXX Timeout value of wouldn't work out, since
+			   the Sink object would fallback to the default timeout value:
+
+			 sink->timeout.connect ? sink->timeout.connect : OSYNC_SINK_TIMEOUT_CONNECT;
+
+			 Really needed?!
+			 */
+
+			osync_objtype_sink_set_connect_timeout(sink, 0);
+			osync_objtype_sink_set_getchanges_timeout(sink, 0);
+			osync_objtype_sink_set_commit_timeout(sink, 0);
+			osync_objtype_sink_set_committedall_timeout(sink, 0);
+			osync_objtype_sink_set_batchcommit_timeout(sink, 0);
+			osync_objtype_sink_set_syncdone_timeout(sink, 0);
+			osync_objtype_sink_set_disconnect_timeout(sink, 0);
+
+			osync_objtype_sink_set_read_timeout(sink, 0);
+			osync_objtype_sink_set_write_timeout(sink, 0);
+		}
+		
+		/* What is meant by this?! Maybe OSyncPlugin.useable?! Not used at all...
+		if (g_getenv("IS_AVAILABLE"))
+			info->functions.is_available = mock_is_available;
+		*/
+
+#endif	
 	}
 
 	osync_trace(TRACE_EXIT, "%s: %p", __func__, env);
@@ -616,7 +829,7 @@ error:
 	return NULL;
 }
 
-static void osync_filesync_finalize(void *data)
+static void mock_finalize(void *data)
 {
 	mock_env *env = data;
 
@@ -625,7 +838,7 @@ static void osync_filesync_finalize(void *data)
 
 /* Here we actually tell opensync which sinks are available. For this plugin, we
  * go through the list of directories and enable all, since all have been configured */
-static osync_bool osync_filesync_discover(void *data, OSyncPluginInfo *info, OSyncError **error)
+static osync_bool mock_discover(void *data, OSyncPluginInfo *info, OSyncError **error)
 {
 	osync_trace(TRACE_ENTRY, "%s(%p, %p, %p)", __func__, data, info, error);
 	
@@ -663,9 +876,9 @@ osync_bool get_sync_info(OSyncPluginEnv *env, OSyncError **error)
 	osync_plugin_set_longname(plugin, "File Synchronization Plugin");
 	osync_plugin_set_description(plugin, "Plugin to synchronize files on the local filesystem");
 	
-	osync_plugin_set_initialize(plugin, osync_filesync_initialize);
-	osync_plugin_set_finalize(plugin, osync_filesync_finalize);
-	osync_plugin_set_discover(plugin, osync_filesync_discover);
+	osync_plugin_set_initialize(plugin, mock_initialize);
+	osync_plugin_set_finalize(plugin, mock_finalize);
+	osync_plugin_set_discover(plugin, mock_discover);
 	
 	osync_plugin_env_register_plugin(env, plugin);
 	osync_plugin_unref(plugin);
@@ -679,9 +892,9 @@ osync_bool get_sync_info(OSyncPluginEnv *env, OSyncError **error)
 	osync_plugin_set_description(plugin, "Plugin to synchronize files on the local filesystem for unit tests");
 	osync_plugin_set_start_type(plugin, OSYNC_START_TYPE_EXTERNAL);
 	
-	osync_plugin_set_initialize(plugin, osync_filesync_initialize);
-	osync_plugin_set_finalize(plugin, osync_filesync_finalize);
-	osync_plugin_set_discover(plugin, osync_filesync_discover);
+	osync_plugin_set_initialize(plugin, mock_initialize);
+	osync_plugin_set_finalize(plugin, mock_finalize);
+	osync_plugin_set_discover(plugin, mock_discover);
 	
 	osync_plugin_env_register_plugin(env, plugin);
 	osync_plugin_unref(plugin);
